package service

import (
	"WorldEpcho/src/config"
	database "WorldEpcho/src/datasource"
	"WorldEpcho/src/models"
	"WorldEpcho/src/utils"
	"bytes"
	"crypto/tls"
	"encoding/json"
	"fmt"
	"github.com/gin-contrib/sessions"
	"io/ioutil"
	"log"
	"net/http"
	"os"
	"path/filepath"
	"strconv"
	"time"

	"github.com/gin-gonic/gin"
)

type RequestBody struct {
	Phone           string `json:"phone"`
	SerialNo        string `json:"serialNo"`
	SchoolName      string `json:"schoolName"`
	CallbackAddress string `json:"callbackAddress"`
	RequestNo       string `json:"requestNo"`
}

//应用id和应用名称缓存结构体
type CacheApp struct {
	AppName             string
	AppID               int64
	UserId              int64
	DpId                int64
	UserInfoTemplate    string
	Phone               string
	SerialNo            string
	Unit                string
	UUID_Val            string
	IsUpdateSelfData    string
	LastEvaluatedResult string
	IsPassMHT           string
}

var (
	//UUID     = utils.GenerateUUID()
	//LoginCacheCode = cache.New(1*time.Hour, 15*time.Second)

	AppCache = make(map[string]*CacheApp)
)

// 发送 POST 请求的函数
func PostSelectAiMentalClinicList(c *gin.Context) {
	/*
	   判断用户是否登录
	*/
	_, isLogin := IsUserLoggedIn(c)
	if !isLogin {
		log.Println("用户未登录")
		c.JSON(http.StatusOK, gin.H{"code": 0, "message": "用户未登录"})
		return
	}
	// 设置请求头
	key := config.Conf.AppKey
	appSecret := config.Conf.AppSecret
	// 获取当前时间
	now := time.Now()
	// 将当前时间转换为Unix时间戳（秒级）
	timestamp := now.Unix()
	timeStr := utils.Strval(timestamp)
	/* 脑机快速录入UUID  */
	var UUID = utils.GenerateUUID()
	//获取token
	token, err := GetPopAccessToken(key, appSecret)
	if err != nil {

	}
	//获取签名
	signature, _ := GenerateSignature(token, timeStr, UUID, appSecret)
	headers := map[string]string{
		"access-pop-token":         token,
		"access-pop-key":           key,
		"access-pop-timestrap":     timeStr,
		"access-pop-noncestr":      UUID,
		"Content-Type":             "application/json",
		"access-pop-signature":     signature,
		"access-pop-supplier-type": "1",
	}
	fmt.Println("请求心理诊疗接口的uuid: ", UUID)
	// 设置请求体
	// 创建一个 RequestBody 类型的实例
	backUrl := config.Conf.CallBackUrl
	//backUrl := "http://127.0.0.1:8089/MentalClinic/data"

	appId := c.PostForm("app_id")
	appName := c.PostForm("app_name")
	appIdInt, err := strconv.ParseInt(appId, 10, 64)
	if err != nil {
		c.JSON(http.StatusBadRequest, gin.H{"error": "Invalid app_id format"})
		return
	}
	// Cache the app_id and app_name
	//cacheMutex.Lock()
	IsUpdateSelfData := c.PostForm("IsUpdate")
	/*
		if IsUpdateSelfData == "" {
			IsUpdateSelfData = "false"
		}
	*/
	//缓存添加应用名称和应用id
	AppCache[UUID] = &CacheApp{
		AppName:          appName,
		AppID:            appIdInt,
		UUID_Val:         UUID,
		IsUpdateSelfData: IsUpdateSelfData,
	}

	//AppCache[UUID] = cacheAppInfo
	/*AppCache[UUID] = CacheApp{
		AppName: appName,
		AppID:   appIdInt,
	}*/
	//cacheMutex.Unlock()

	/*
	  手机号和学号+学校名称，二选一
	*/
	phone := c.PostForm("phone")
	if phone != "" {

		AppCache[UUID].Phone = phone
		requestBody := RequestBody{
			Phone:           phone,
			CallbackAddress: backUrl,
			RequestNo:       UUID,
		}
		// 将结构体实例转换为 JSON 格式的字节流
		jsonData, err := json.Marshal(requestBody)
		if err != nil {
			fmt.Println("Error marshalling JSON:", err)
			return
		}
		url := config.Conf.MentalClinicUrl
		// 创建请求
		req, err := http.NewRequest("POST", url, bytes.NewBuffer(jsonData))
		if err != nil {
			c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
			return
		}
		// 添加请求头
		for key, value := range headers {
			req.Header.Set(key, value)
		}

		// 发送请求
		client := &http.Client{}
		resp, err := client.Do(req)
		if err != nil {
			c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
			return
		}
		defer resp.Body.Close()

		// 读取响应体
		body, err := ioutil.ReadAll(resp.Body)
		if err != nil {
			c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
			return
		}

		var result map[string]interface{}
		err = json.Unmarshal(body, &result)
		if err != nil {
			c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
			return
		}

		code := int(result["code"].(float64))

		// 将响应体作为字节切片发送回客户端
		//c.Data(resp.StatusCode, "application/json", body)
		if code == 200 || resp.StatusCode == 200 {
			fmt.Println("响应体: ", body)

			// 异步执行ProcessData逻辑
			go func(uuid string) {
				// 延迟1秒以模拟异步行为
				time.Sleep(1 * time.Second)

				// 创建跳过证书验证的Transport
				tr := &http.Transport{
					TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
				}

				// 准备模拟数据
				// 使用fmt.Sprintf拼接字符串，其中包含uuid
				mockDataTail := fmt.Sprintf(`"requestNo": "%s"}`, UUID)
				mockData := `{
	"baseInfos": {
		"evaluateWaveformAdvantages": [
			{
				"createBy": "13298598527",
				"createTime": 1673423384000,
				"description": "对自身能力的信心。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/8087b86752b444a29acc1e54948d469c.png",
				"id": 164,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 1,
				"status": 0,
				"type": "poc",
				"typeStr": "1自信力",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423408000,
				"description": "* 无论遇到什么困难，都能坚持不懈的完成任务，在任何困难条件下都能长时间保持高水平投入状态。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/7c8bb5df94964d258f542aec895b303a.png",
				"id": 165,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 2,
				"status": 0,
				"type": "wp",
				"typeStr": "2毅力",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423434000,
				"description": "* 即自我控制的能力，指一个人对自身的冲动、感情、欲望施加的正确控制。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/a7733def606b4677a54b3a22cafd8548.png",
				"id": 166,
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 3,
				"status": 0,
				"type": "honest",
				"typeStr": "3自控力",
				"updateBy": "13298598527",
				"updateTime": 1673875267000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423464000,
				"description": "* 勇于承担责任。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/5e77e8e89e7c4f1f9c941568ed912ccc.png",
				"id": 167,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 4,
				"status": 0,
				"type": "sor",
				"typeStr": "4责任感",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423504000,
				"description": "* 在压力下，在行动的最后期限之前及时做出决策。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/19356f9cb7784dcd8141aa07dde9222f.png",
				"id": 168,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 5,
				"status": 0,
				"type": "at",
				"typeStr": "5果断性",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423532000,
				"description": "* 评估问题解决情境下的风险，可能的结果以及可能造成的反响、仔细权衡行动方案，在行动上要有耐心。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/33361823a8ed44559acac1de87c30aa3.png",
				"id": 169,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 6,
				"status": 0,
				"type": "aoj",
				"typeStr": "6判断能力",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423569000,
				"description": "* 利用手中资源创造性的解决问题。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/0237a5795b9e4dc0811f64f76654ba5f.png",
				"id": 170,
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 7,
				"status": 0,
				"type": "ps",
				"typeStr": "7问题解决",
				"updateBy": "13298598527",
				"updateTime": 1673875267000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423597000,
				"description": "* 有效的沟通，与他人沟通良好。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/5d75b2f9f98e4a5b98391213ec49c5ed.png",
				"id": 171,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 8,
				"status": 0,
				"type": "atc",
				"typeStr": "8沟通能力",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423623000,
				"description": "* 情绪富有弹性，能忍受困难和挫折，在紧急时刻或者压力之下依然保持战斗力。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/9a0817c2479645efa51e4f9846ff3132.png",
				"id": 172,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 9,
				"status": 0,
				"type": "apa",
				"typeStr": "9抗压能力",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423644000,
				"description": "* 自我约束与自我调节能力。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/b58b3cf4e5c8402287091c6c6f1fc068.png",
				"id": 173,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 10,
				"status": 0,
				"type": "sd",
				"typeStr": "10自律",
				"updateBy": "13298598527",
				"updateTime": 1673875280000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423671000,
				"description": "* 将群体目标至于个人目标之上，荣辱与共。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/c9d1f69a6a954a8f8c8f58f501783d45.png",
				"id": 174,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 11,
				"status": 0,
				"type": "cpa",
				"typeStr": "11合作",
				"updateBy": "13298598527",
				"updateTime": 1673875280000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423698000,
				"description": "* 立即改变需求的行动，修改计划以适应形势。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/68cf394014ea4c9d8194c4b7ecf14c9f.png",
				"id": 175,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 12,
				"status": 0,
				"type": "apb",
				"typeStr": "12适应力",
				"updateBy": "13298598527",
				"updateTime": 1673875280000,
				"waveformRank": "13"
			}
		],
		"evaluateWaveformCapacitys": [
			{
				"advise": "如何提高共情能力？ 1.学会换位思考：能从对方角度为对方的行为寻找合理性，以最大限度地理解对方。  2.学会倾听：指能全身心地聆听对方的表达。倾听不仅指听取其口语表达的内容，还包括观察非语言的行为，如动作、表情、声音语音语调等。  3.表达尊重：尊重对方的个性及能力而不是凭自己的感情用事。接纳对方的信念和所做出的选择或决定，而不是评论或试图替其做决定。",
				"createBy": "13298598527",
				"createTime": 1673505354000,
				"description": "共情能力是一种将自己置于他人的位置、并能够理解或感受他人在其处境里所经历的事物的能力。",
				"id": 145,
				"knowledge": "什么是共情能力指能够站在对方立场设身处地思考、于人际交往过程中能够体会他人的情绪和想法、理解他人的立场和感受，并站在他人的角度思考和处理问题。它主要体现在情绪自控、换位思考、倾听能力以及表达尊重等与情商相关的方面。共情之所以是利他行为的基础，是因为具备共情特质的人能切身感受到别人的需要与苦恼，并能在必要时以得体和尊重的方式向他人提供支持与帮助，因此，非常有助于健康人际关系的建立。日常生活中，有共情能力的人较少与别人发生冲突，因为他（她）总能最大限度地理解别人，并以平和的心态与人相处。共情（同理心）与同情心的区别同情心，是认知到别人的痛苦，从而引起恻隐之心。共情能力是能够感同身受，设身处地为他人着想，实际感受到他人的痛苦。当听到他人遭遇不幸时，有同情心的人会说：“我真的替你感到难过。”但是有共情能力的人却会这样说：“我也遇到过这样的事，我知道这是什么样的感觉。”同理心是牵涉两个人的互动过程，必须认真地倾听，而且了解、接纳对方的感觉，这是一种认同他人体验的态度。同理心是一种关心与爱的艺术，是高情商的表现。共情能力的影响因素遗传：有研究表明，遗传在共情中的影响可能达到30%以上。性别：多项研究证实，女性共情能力普遍要高于男性。",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 1,
				"status": 0,
				"type": "ec",
				"typeStr": "1共情能力",
				"updateBy": "13298598527",
				"updateTime": 1673875339000,
				"waveformRank": "13"
			},
			{
				"advise": "攻击性行为在儿童时期的表现   孩子在成长过程中会逐渐学习控制和表达自己的情绪和行为、以积极的方式与他人沟通。在学会之前，孩子可能会试图用攻击性行为（打、踢、咬、抓等）来表达自己。攻击性行为的原因有很多，如表达需要关注、快速达到自己的目的、在行为中获得力量感等。",
				"createBy": "13298598527",
				"createTime": 1673505376000,
				"description": "暴力行为，即攻击性行为，指身体、语言或心理上导致人或物受到伤害的行为（如打、踢、咬、抓、威胁、羞辱、责备、破坏等）。抗暴力行为能力指的是阻止或限制攻击性行为的能力。",
				"id": 146,
				"knowledge": "攻击性行为在儿童时期的表现   孩子在成长过程中会逐渐学习控制和表达自己的情绪和行为、以积极的方式与他人沟通。在学会之前，孩子可能会试图用攻击性行为（打、踢、咬、抓等）来表达自己。攻击性行为的原因有很多，如表达需要关注、快速达到自己的目的、在行为中获得力量感等。影响攻击性行为的因素 1.生物因素：男性比女性具有更多的身体攻击性行为，而女性则具有更多非身体攻击性行为（如言语攻击）。除了性激素之外，肾上腺皮质激素和垂体促肾上腺皮质激素对攻击行为也有调节作用。 2.疾病因素：癫痫、痴呆症、自闭症、注意力缺陷多动障碍、精神分裂症和脑损伤等均可导致攻击性行为增加。 3.其他因素：药物酒精滥用、不健康的家庭关系、童年目击进攻性行为和慢性压力等均可导致攻击性行为增加。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 2,
				"status": 0,
				"type": "rtv",
				"typeStr": "2抗暴力行为能力",
				"updateBy": "13298598527",
				"updateTime": 1673855161000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提高情感表达能力？ 1.通过写日记、读故事、看电影、接触艺术作品等方式，增加自己与情绪接触的机会。 2.使用语言、文字等方式表达自己的内心感受。 3.多倾听、换位思考，提高对他人的理解，表达对他人的关注。 4.若感觉自己受述情障碍影响较大，可以寻求心理咨询师的帮助。",
				"createBy": "13298598527",
				"createTime": 1673505395000,
				"description": "情感表达能力弱的人格特质也被称为述情障碍，表现为缺乏情感意识、难以识别和表述感受和情绪。",
				"id": 147,
				"knowledge": "述情障碍有什么表现？述情障碍，即情感表达能力弱，表现为对于情绪的识别不敏感，可能会看起来比较冷漠、有距离感。有述情障碍特质的人，可能会对积极的情绪（如愉悦、惊喜）无动于衷，也可能会面对消极情绪（如紧张、焦虑）异常平静。另一方面，虽然大脑处理情绪的能力较弱，身体的感官却不会消失甚至会变得更敏感，如在面对压力的时候可能会肩膀僵硬。影响述情障碍的因素 1.社会文化差异：不同文化背景下的人会表现出不同水平的述情障碍，如亚裔美国人的述情障碍水平高于欧裔美国人。 2.社会经济地位：受教育程度低和贫困的人更容易出现高述情障碍，因为自身的社会经济状况要求他们不得不为了生存而努力，不鼓励他们情绪和情感的加工和表达。 3.家庭环境：不良的家庭环境，可能会影响儿童早期社会化、压抑对依恋的感受意识，是述情障碍发展的危险因素。与述情障碍相关的疾病精神疾病（孤独症、抑郁症、精神分裂症等）患者大多存在述情障碍。糖尿病、冠心病、高血压、脑卒中、银屑病、哮喘、十二指肠溃疡、乳腺癌等疾病患者较普通人群受述情障碍影响比例更高。",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 3,
				"status": 0,
				"type": "aea",
				"typeStr": "3情感表达能力",
				"updateBy": "13298598527",
				"updateTime": 1674098238000,
				"waveformRank": "13"
			},
			{
				"advise": "如何克服社交恐惧？ 1.找出原因尝试克服：找出具体触发你恐惧或者焦虑的因素，以及你害怕的场景。根据你分析的触发因素和惧怕场景，通过了解什么是导致你焦虑的反应，可以以更积极或有针对性的方式来处理这些焦虑情绪。 2.尝试让自己平静的方法：学会识别自己对焦虑或者恐惧的身体反应，然后尝试那些让自己平静下来的方法：比如深呼吸，或者将身体收紧三秒再放松，听一些能让你心情愉悦的歌，或者静下心来冥想等。 3.改善自己的想法：用积极的想法来代替那些大脑自动产生的消极想法，培养更多的自信。 4.减少对自己的注意力：观察你周围的人和环境，将专注点放在别人正在说的话上，而不是过多专注于自己。 5.尝试多与理解你的人沟通：试着去寻找那些与你有同样烦恼，理解你的人，并与他们多交流。也许可以帮助你们互相改善，或者让你积累一些自信。他们的一些成功经历也能成为你借鉴的经验。",
				"createBy": "13298598527",
				"createTime": 1673505427000,
				"description": "抗社交恐惧能力是指抵抗因存在与社交场合而产生焦虑、惊恐的能力。",
				"id": 148,
				"knowledge": "什么是社交恐惧社交恐惧是一种对任何社交或公开场合感到强烈恐惧或忧虑的心理状态。社交恐惧的表现具有社交恐惧症的人在陌生人面前或可能被别人仔细观察的社交或表演场合，有一种显著且持久的恐惧，害怕自己的行为或紧张的表现会引起羞辱或难堪。甚至有些人对参加聚会、打电话、到商店购物、或询问权威人士都感到困难。产生社交恐惧的原因 1.生理原因：有研究表明可能是由于人体内被称为“5-羟色胺”的化学物质失调所致，这种物质负责向大脑神经细胞传递信息，其过多或过少都可能引起人们的恐惧情绪。 2.心理原因：社交恐惧症患者一般自尊心较强，害怕被别人拒绝，或者对自己的外貌没有信心。 3.家庭原因：从小性格受到压抑，或者是父母没有教会他们社交的技能，或者是家庭搬迁过于频繁。 4.社会原因：本身所处的社会环境较为恶劣，与人交往时受到的挫折居多。 5.思维方式：不正确的思维方式造就了社交恐惧症，比如过分的完美主义者受到打击后带来的过分自身反省，每个社交恐惧症患者都应该对“自己”有充分的了解。 6.遗传因素：研究发现恐惧症具有家族遗传倾向，尤其影响到女性亲属。",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 4,
				"status": 0,
				"type": "ai",
				"typeStr": "4抗社交恐能力",
				"updateBy": "13298598527",
				"updateTime": 1673875339000,
				"waveformRank": "13"
			},
			{
				"advise": "如何降低孤独感？ 1.走出家门，感受室外的空气和阳光，和周围的人打个招呼。 2.做些锻炼，慢跑、游泳、瑜伽、拳击都是不错的选择。 3.给家人或朋友打个电话、闲聊一会，约个时间出来见面。 4.控制手机、电脑、电视等屏幕使用时间。 5.试着养一个宠物。 6.规律作息，睡个好觉。 7.了解自身，知道孤独不是永久的状态。 8.如果以上方法都不奏效，寻求心理医生或咨询师的帮助。",
				"createBy": "13298598527",
				"createTime": 1673505444000,
				"description": "孤独指的是社交需求没能得到满足时的一种心理感受。抗孤独能力强的人更不容易感到孤独。",
				"id": 149,
				"knowledge": "经常感到孤独正常吗？人人都会感到孤独，性别、年龄、样貌、社会经济条件等都不能让你免于孤独，对此不用过分担心。孤独与独处状态不能完全等同，孤独可能源于长时间的独处，或者仅仅是感到不被他人理解。孤独可能造成的影响孤独会对心理和身体造成一系列的影响：孤独让人更容易感到抑郁、睡眠质量下降、执行功能受损、认知能力下降、免疫力下降、心血管功能下降；孤独是阿兹海默病、心脏病、2型糖尿病、关节炎等疾病的风险因素；孤独会提高“战斗或逃跑反应”（fight-or-flightresponse，当人面对威胁时的一系列生理反应）；孤独还会影响人的寿命。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 5,
				"status": 0,
				"type": "ala",
				"typeStr": "5抗孤独能力",
				"updateBy": "13298598527",
				"updateTime": 1673854765000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提升积极情感？   记录你的积极情感：记录下你在生活中已经经历且熟悉的积极情感。当你发现有新的积极情感时，首先注意，直到你能确认时再记录下来。当记录到一定数量时，看着名单，回忆每一个情感产生时的活动、情景或人物。 关注一种积极情感并且加强它：回忆该积极情感产生时的情况，尝试将这些场景增加到日常生活中，以此加强该类型的积极情感。创造积极情感的宝箱：有时候我们会忘记曾经积极情感的来源，因此积极情感的宝箱变得很有意义。可以将那些产生积极情感的物品保存下来，比如照片、纪念品、日记本、卡片等。将它们收藏在你的宝箱里，你可以随时增加、减少、整理它们。当你开心或不开心的时候，将宝箱打开，让它们增加你的积极情感。",
				"createBy": "13298598527",
				"createTime": 1673505460000,
				"description": "积极情感，又叫积极情绪或正性情绪，是由于体内外刺激、事件满足个体需要而产生的伴有愉悦感受的情绪。",
				"id": 150,
				"knowledge": "对于积极情感的描述罗素：“积极情感就是当事情进展顺利时，你想微笑时产生的那种好的感受”。 Fredrickson：“积极情感是对个人有意义的事情的独特即时反应，是一种暂时的愉悦”。孟昭兰：“积极情感是与某种需要的满足相联系，通常伴随愉悦的主观体验，并能提高人的积极性和活动能力”。十种人们最常感受到的积极情感形式 1喜悦（Joy）当我们感受到喜悦时，我们一般处于这样的情形下：一切按照预定的方式发展，结果符合我们的期待，甚至比我们所期待的要更好。喜悦是一种轻快而明亮的感觉，当我们感到喜悦，我们会感到浑身轻松，甚至周围的事物看起来也更生动、顺眼，我们可能会想加入他人的谈话、对接下来的事跃跃欲试。 2感激（Gratitude）当我们意识到他人对我们的付出，我们会体验到感激。有时我们感激的对象也不一定是人，也可能是感受到某种事物带给我们益处。当我们赞赏人、事、物的可贵，感激就出现了。感激会带给我们“想要付出回报”的冲动，我们会希望对帮助过我们的人做点好事，也可能会想通过帮助其他人来把自己受过的恩惠传递出去。 3宁静（Serenity）宁静是一种绵柔、低调、放松版本的喜悦，通常发生在感觉深处安全而美好的环境中。宁静会让人们更加需要沉浸在当下，去品味当前的感受。 4兴趣（Interesting）兴趣是我们在安全的环境中，被一些新颖的人、事、物吸引了注意力时感受到的情绪。我们会被兴趣牵引着，去探索、尝试，去消除神秘、了解更多。 5希望（Hope）相比平淡的日常，往往在事情发展对我们不利或者存在不确定性时，我们更容易感受到希望。希望的核心是我们相信事情能好转、好事有可能发生的信念和愿望。因为相信未来还有可能是好的，我们才愿意继续生存下去。 6自豪（Pride）自豪是一种因为我们是、我们做了、或我们达到了什么而产生的满足感。这是一种很重要的感情，因为它提高了我们的自尊，让我们对自己更有信心，并且使我们面对负面情绪时更加坚强。 7逗趣（Amusement）生活充满了幽默的故事或玩笑，并且那些有趣的经历会让你开怀大笑，真正的感受到被逗趣了。 8激励（Inspiration）当我们看到完美的杰作、或者听到伟大的演讲，会因此而产生被激励的感觉。它是一种可以改变心情的非常强烈的驱动力量，让人去追逐完美。 9敬畏（Awe）是一种面对崇高、狞厉、神秘事物所体验到的惊奇、畏惧却又爱戴、崇敬的感受。在对自然、社会、艺术的审美中，都可能产生。 10爱（Love）是最为复杂的情感。当我们还是刚出生的婴儿时，第一体验到如此广泛而又强烈的情感。最纯净的形式是无条件的爱，只有付出而不求回报。积极情感的影响：积极情感带来开放性，提升我们的创造力。积极情感带来积极的人际互动体验。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 6,
				"status": 0,
				"type": "pac",
				"typeStr": "6积极情感能力",
				"updateBy": "13298598527",
				"updateTime": 1673854980000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提高执行能力？有研究表明，通过专业的针对性行为训练可以提高执行能力，包括任务转换训练、记忆更新训练、反应抑制训练等。积极参加有氧运动。参与有氧运动可以有效改进大脑前额皮质功能；同时，人体在有氧运动过程中会分泌内啡肽，可以增加愉悦程度，多人参与的有氧运动还能增加社会联结，而悲伤、孤独等负面情绪会带来执行能力的降低。",
				"createBy": "13298598527",
				"createTime": 1673505474000,
				"description": "执行能力是指完成特定的目标或者任务时，需要动用的一些高级的认知功能和过程，如专注力、工作记忆力、控制力、认知灵活性等。其本质是对认知过程进行控制和调节的能力，一般也称作执行功能。",
				"id": 151,
				"knowledge": "什么是执行能力  执行能力，或者叫执行功能（Executive function），是人们在执行任务，完成具体的目标时需要的有意识的心理控制能力，是一种非常综合、高级的认知功能。执行能力主要体现在以下几个方面：计划和决策能力：能分析需要完成任务需要的步骤，并且构建完整的方案。管理能力：能对时间、资源进行合理的安排。工作记忆能力：在完成复杂的工作的时候，能够记住其中关键的信息，并且利用这些信息制定解决问题的计划。控制力：能在行动时抑制本能的冲动，以完成任务为目标，不拖延，在完成的过程中也能控制自己的行为，管理自己的情绪，并且不被其他事物分散注意力。适应能力：在面对新的变化、阻碍、发生错误的时候能及时修改计划，对环境的变化能有足够的适应。执行能力的发展过程执行能力是成熟的最晚的心理功能之一，在人生的各个阶段都处于不断发展和变化中。 幼儿期：注意力、工作记忆力等能力在幼儿时期就会出现。在幼儿园时期（3～5岁），适应能力、计划性等能力也会开始快速形成。当然，在这个时期，还不具备成熟完整的执行能力，所以在幼儿期经常会犯错误，这不是能力不足导致的，而是由于大脑的决策能力、执行能力在这个时期还不够成熟。青春期前：这个时期，执行能力会再有一次快速提升，尤其是语言工作记忆力，有目的的行为、注意力、控制力等能力有显著提升。不过和幼儿期相似，整体的执行能力还处于比较有限的状态。青春期：青春期之前的出现的能力在这个阶段继续保持增长，并且系统的整合起来。专注能力可能在这个阶段会有一个爆发式增长。成年期：成年后的最重要的变化就是前额叶皮层的持续髓鞘化。20-29岁是执行能力最强的一个时期，使得大多数人在这个时期可以参与非常具有挑战的工作。年龄逐渐增大后，执行能力也会慢慢有相应的下降。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 7,
				"status": 0,
				"type": "ate",
				"typeStr": "7执行能力",
				"updateBy": "13298598527",
				"updateTime": 1673854790000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提高抗职业倦怠能力？ 1.通过积极的沟通和反馈，获得同事或上级的情感支持，调整来自工作中的各类压力源。 2.平衡工作和生活时间，保证充足的睡眠。",
				"createBy": "13298598527",
				"createTime": 1673505511000,
				"description": "抗职业倦怠能力是指当工作压力大等职业因素出现时，可以正确克服身体、情绪或精神疲惫状态的能力。",
				"id": 152,
				"knowledge": "",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 8,
				"status": 0,
				"type": "rtb",
				"typeStr": "8抗职业倦怠能力",
				"updateBy": "13298598527",
				"updateTime": 1673855019000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提高主观幸福感？ 1.维护良好的人际关系：幸福的人更可能被别人选做朋友和信任的对象，更多的去帮助别人，减少消极情绪，建立互信关系满足人的归属需要，会令人感到幸福和满意。 2.创造满意的居住环境：我们要选择人身安全、让自己和家人舒适，有优美自然风光、气候舒适、可以选择能欣赏到音乐和艺术的地方居住。 3.积极增加体育锻炼：锻炼的短期效果导致大脑产生内啡肽和类吗啡的释放；而长期有规律的锻炼减少了抑郁和焦虑，提高了我们工作的准确性和速度，提升了我们的自我概念，促进了心血管的健康和机能。另外，有规律的锻炼的人经常和别人结伴运动，也能在这种额外的社会支持中受益，因此我们应该保持良好的健康状况，定期参加体育锻炼。 4.适度的业余活动：有研究表明，参加休闲或运动组织，尤其是那些包含舞蹈、音乐、义工或运动组织，可以带来更高的主观幸福感。",
				"createBy": "13298598527",
				"createTime": 1673505535000,
				"description": "主观幸福感指个体根据自定的标准对其生活质量的整体性评价，它是个体对于自己是否幸福的一种主观感受。",
				"id": 153,
				"knowledge": "",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 9,
				"status": 0,
				"type": "swb",
				"typeStr": "9主观幸福感",
				"updateBy": "13298598527",
				"updateTime": 1673875339000,
				"waveformRank": "13"
			},
			{
				"advise": "如何克服情绪化进食？找到引发情绪化进食的原因：想要停止情绪化进食，就要正视它，剖析在什么诱因下自己开始进食，如果这一模式相对清晰，改正起来也相对容易。转移注意力：把心思从吃上暂时移开，想吃东西时，尝试让注意力分散10分钟，比如立刻起身去做别的事情（收拾屋子/去锻炼/洗个脸敷个面膜）或者跟朋友聊天打电话等，让吃的念头被分散。准备健康食物：在手边准备好健康的食物，比如即食燕麦粥、速溶番茄汤、无糖全麦面包等相对健康又有饱腹感的食物，当出于心理需要想吃东西的时候，拿健康的食物来满足对情绪化饮食的迫切需要。培养应对情绪问题的习惯：可以尝试找一些替代食物的方式，比如压力大的时候，去洗个热水澡或者去做按摩；感到抑郁孤独的话，试试养一只宠物或者定期与好友聚会或网上交流；或者把某项运动发展为自己的爱好，比如瑜伽。养成健康的生活习惯：早睡早起不熬夜，只有当睡眠质量高、身心放松，休息充分时，才能更好地处理生活中的困难以及应对多变的环境；规律且适度的运动，定期减压、放松，同时进行一些积极地社交活动，都有助于改善情绪化进食。",
				"createBy": "13298598527",
				"createTime": 1673505566000,
				"description": "抗情绪化暴食能力是指对抗由于情绪方面的原因而过度进食的能力。",
				"id": 154,
				"knowledge": "什么是情绪化进食情绪化进食是为了满足情绪而进食，而非因为生理性饥饿而吃。压力会导致人体皮质醇水平升高，而高碳水，高糖和高脂肪的食物可以降低压力荷尔蒙，在沮丧和不安的时候拼命吃高热量食物会减轻心理压力和负面情绪,然而食物入口的快感仅能维持1分钟,接下来是吃完之后懊悔、羞愧、自责的心理，肥胖也随之产生。因此，情绪化进食并不能解决人们实际生活中的问题和情绪,反而会加剧人们情绪的不稳定性，如果不加控制的话，情绪化进食有可能会演变成暴食症或神经性贪食症。你是一名情绪化进食者吗？当你感到紧张的时候是否吃的更多？当你并不感到饥饿或者已经饱足的时候是否进食？当你感到伤心、愤怒、无聊、焦躁时通是否过进食来抚慰自己、恢复平静，通过进食来让自己感觉更好？你是否用食物来奖励自己？你是否经常吃到过撑？你是否对食物感到无力或者不受控制？情绪化饥饿与生理化饥饿的不同情绪化饥饿突如其来：情绪化饥饿突如其来，它在一瞬间让你觉得不可抗拒而又十分迫切；生理性饥饿是逐渐而来，想吃的愿望并不十分迫切，也不会要求立即的满足（除非你已经很久没有吃东西了）。情绪化饥饿渴求特定的食物寻求安慰：情绪化饥饿渴求特定的食物寻求安慰；当你处于生理性饥饿时，几乎所有的食物听起来都很好吃——包括例如蔬菜的健康食品。情绪化饥饿往往导致暴食：当你意识到之前，你已经吃了一大袋薯片或者一整盒冰激凌，却并没有真正地注意到什么或者充分地享受食物。而当你因为生理性饥饿而进食时，往往更加能够意识到自己在做什么。情绪化饥饿并不会因为吃饱而得到满足：你只会想吃得越来越多，往往直到撑得很不舒服的时候才会停下来。另一方面，生理性饥饿并不会导致你吃撑，当你的胃填满的时候，你就感到了满足。情绪化饥饿往往导致后悔、内疚和羞愧：情绪化饥饿往往导致后悔，内疚和羞愧。当你因为生理性的饥饿去吃东西时，不太可能感到内疚和羞愧，因为你仅仅是给予身体所必需的东西。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 10,
				"status": 0,
				"type": "rteo",
				"typeStr": "10抗情绪化暴食能力",
				"updateBy": "13298598527",
				"updateTime": 1673855127000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提高组织管理能力管理者的组织管理能力，除了体现在建立机构、团队建设之外，还充分体现在个人的综合素质与综合能力之上。个人组织管理能力是管理者学习能力、判断能力、决策能力、实施能力、协调能力等的集合，不是单一的某种能力的体现。所以，提高管理者的组织管理能力，必须从多方面努力，从多种能力的提高人手。(一)增强学习能力从书本上学习，从实践中学习，勤于思考。1、根据工作要求及个人情况，制订出学习内容、目标、进度等的学习计划。有所选择，有所规划，树立志向、志气、志趣，长期坚持，学有所为，有的放矢。2、善于利用时间、讲究学习方法。3、提高学习效率、学以致用、勤于思考、建立自己的体系；(二)增强判断能力，对事物有自己清晰明确的判断标准，思路清晰，能更快地作出准确无误的判断。提高判断能力既需要理论上加以训练，又需要在工作实践中不断提高。1、正确掌握运用各种类型的判断逻辑形式等必要知识；2、加强个人思想修养，不唯书、不唯上、有自信、敢创新，无私、无畏；3、努力参与社会实践，在实践中提升。4、不轻易判断他人，要客观判断自己，在管理工作中不能以貌取人，仅凭第一印象就对一个人下判断。要听其言，观其行。而对待自己，则要有自知之明，不能过于自信，自大狂妄。也不能过于悲观，缺乏自信。(三)增强决策能力，决策能力是组织管理能力的核心。1、提高决策素质是首要，决策者要作出正确的决策，必须不断提高自身的思想品质修养、知识能力修养、性格气质修养、情绪情感修养等。2、掌握决策原则，决策的基本原则有求实原则、系统原则、民主原则、可行原则、创新原则等。求实原则要求领导者决策符合客观实际；系统原则要求领导者决策必须在综合一切信息的基础上考虑各种情况并提出相应的解决方案；民主原则要求决策者必须按照科学、透明的程序在集体智慧的基础上作出决策；可行原则要求领导者决策是可以在实践中实施的；创新原则要求领导者决策必须以新的思路去解决新的问题。3、优化决策程序，领导者在决策时，不仅要按照一般程序来进行，而且要尽可能根据科学高效原则减少一切不合理环节，在决策过程中要注意优化每一个环节，在各种方案中进行优选。4、掌握科学的决策方法，决策是有方法可循的，按照不同的决策类型，应当选择不同的决策方法，常见的可分为定性决策和定量决策两大类。领导者可根据情况适当选择或综合使用，要在实践中体会各种决策方法的优缺点，对其中一些适合自身的决策方法能够运用自如。(四)增强协调能力，领导协调是组织管理工作中经常性的工作。包括协调与上级的关系、与下级的关系、与同级的关系，各种关系的协调不可有固定的模式，要因势利导，针对不同对象在方法上有所不同，有所选择。基本的方法有:1、疏通阻塞：各种不协调的现象的出现，大部分是因为信息阻塞引起的。领导者要善于疏通阻塞。重大问题先协商后再决策，一般问题先通气后再执行，暂时有分歧的就冷处理，善于等待，疏通关节2、多商量：商量是协调最重要的环节。领导者要用平等的态度对待别人，不要强加于人。既然是商量，无论对方和自己的关系如何，地位如何，都要把对方摆在平等的地位。3. 化解矛盾：领导者无时无处不生活在矛盾中，诸如利益冲突、心理冲突、角色冲突、职权冲突、目标冲突等。要正视矛盾，要善于化解矛盾。可采用以冷制热法、含糊处理法、春风化雨法、接受时间法、彼此退让法等。4、保持平衡：各种社会关系需要互相依赖，竞争与合作是一种平衡。领导者要从总体上把握平衡，在坚持原则的前提下追求平衡，从动态中寻求平衡。要团结所有团队成员形成一个相与配合的组织，形成和谐的人际关系。(五)增强执行能力，好的想法、规划、计划，只有付诸于良好的执行才能够产生好的结果；",
				"createBy": "13298598527",
				"createTime": 1673505584000,
				"description": "组织管理能力是指为了有效地实现目标，灵活地运用各种方法，把各种力量合理地组织和有效地协调起来，为了实现目标不断优化人、事、物等的能力。包括协调关系的能力和善于用人的能力等等。",
				"id": 155,
				"knowledge": "",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 11,
				"status": 0,
				"type": "atom",
				"typeStr": "11组织管理能力",
				"updateBy": "13298598527",
				"updateTime": 1673854813000,
				"waveformRank": "13"
			}
		],
		"evaluateWaveformEmotions": [
			{
				"advise": "缓解焦虑的建议1、人如果抗焦虑能力弱，容易出现焦虑情绪，建议学会正确处理焦虑情绪；2、除了遗传因素，这类人也更容易出现焦虑情绪：性格自卑个性悲观的人、抽烟酗酒的人、遭遇压力和创伤的人；3、适度的焦虑可以使我们获得动力去做事，过度的焦虑却会使我们情绪痛苦，还会带来很多身体不适；4、使用以下方法可改善焦虑状态：a在安静的地方进行深呼吸、松弛肌肉和想象积极的画面（如静谧的自然景观）；b补充L-酪氨酸和维生素B12有利于舒缓情绪；   c调整到健康的生活习惯；   * 如需使用膳食补充剂，或需要心理干预，建议您咨询权威的健康专家或临床医生。",
				"createBy": "13298598527",
				"createTime": 1673509657000,
				"description": "反映人的焦虑程度，指数越高，人越焦虑。",
				"id": 80,
				"knowledge": "焦虑是什么 焦虑，是包含了担心、紧张、恐惧、尴尬等情绪的一种情绪的统称，它同时还带来了一些躯体上的生理唤醒，比如心跳加快、呼吸急促、手心出汗、眩晕、肠胃不适等躯体反应。焦虑完全是坏事吗适度的焦虑可以使我们获得动力去做某事，对于焦虑和绩效的研究得出的倒U型曲线也证实这一点，中等水平的焦虑可以得到最高的绩效。然而，过度的焦虑会带来情绪上的痛苦，还会带来很多躯体上的不舒服，比如会经常觉得累、肌肉紧张、脖子酸疼，睡眠质量下降、幸福感降低等。总之，过度焦虑会让人的生活质量下降很多。焦虑的影响因素遗传：从遗传学上来说，同卵双胞胎，如果一人患病，另一人患病的几率也会增加。如果父母中有焦虑症，子女患病的概率会上升到50%~75%。性格：一些个性上的特质，如自卑、过于依赖别人、自责、悲观、容易被压力击垮等，都会使你更容易产生焦虑。抽烟、酗酒：有研究表明，约有30%的严重焦虑症患者与酗酒以及使用滥用物有关。此外，焦虑症患者对尼古丁上瘾的机率比正常人高出约2倍。环境：令人感到有压力的生活事件及失落感可能诱发焦虑，如丧偶、离婚、失业、财务危机、出现不良健康状况等。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 1,
				"status": 0,
				"type": "ani",
				"typeStr": "1焦虑指数",
				"updateBy": "13298598527",
				"updateTime": 1673855557000,
				"waveformRank": "13"
			},
			{
				"advise": "几个改善睡眠的建议：1、身心放松：保持平和的心态，平时避免情绪过度波动，避免精神紧张、焦虑、抑郁的情绪。睡觉前可以听舒缓的音乐，让全身心得到充分放松，避免情绪激动。也可以通过睡前泡脚来促进血液循环，从而达到放松的状态，更容易进入深度睡眠；2、合理饮食：入睡前避免饥饿或者饮食过饱，避免饮酒、咖啡、浓茶等刺激性强的饮料；3、良好作息习惯：尽量早睡早起，午睡时间不宜过长，建立良好的生物钟，容易进入深度睡眠；4、睡眠环境良好：睡眠时寝室光线宜暗，周围环境安静，温度和湿度适宜，温度在22℃-26℃，湿度在40％-60％为宜。5、在日常生活中可以多晒晒太阳，在白天时多做体力活动或者体育锻炼，可以让身体处于略微疲劳的状态，提高睡眠深度。6、也可以吃药食同源的物质，比如酸枣、大枣可以起到安神的作用，提高睡眠质量。",
				"createBy": "13298598527",
				"createTime": 1673509678000,
				"description": "反映人的睡眠质量情况，指数越高，睡眠质量越好。它和人的深度睡眠时间、睡眠深浅、多梦、晚睡等相关。",
				"id": 81,
				"knowledge": "什么是深度睡眠深度睡眠是人体入睡以后大脑不进行活动的深度休息，是人体睡得最熟、最香的阶段，此时人听不到外界的声音，也极难被别人唤醒。人的机体进入睡眠状态后是先进入浅睡眠，然后自然进入深睡眠，然后又是浅睡眠，然后再是深睡眠，深睡眠和浅睡眠来回交替直到醒来。深度睡眠被称作是“黄金睡眠”，约占整个睡眠时间的25％，有助于人身体和细胞的修复，恢复人的脑力、体力和精力；深度睡眠也是孩子身体长高、智力发育的重要时间。",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 2,
				"status": 0,
				"type": "sli",
				"typeStr": "2睡眠指数",
				"updateBy": "13298598527",
				"updateTime": 1673875397000,
				"waveformRank": "13"
			},
			{
				"advise": "",
				"createBy": "13298598527",
				"createTime": 1673509708000,
				"description": "反映处理问题不被情绪和感觉所左右，理性对待的程度，指数越高人越理性。",
				"id": 82,
				"knowledge": "什么是理性思维理性思维是一种有明确的思维方向，有充分的思维依据，能对事物或问题进行观察、比较、分析、综合、抽象与概括的一种思维。是一种建立在证据和逻辑推理基础上的思维方式。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 3,
				"status": 0,
				"type": "rai",
				"typeStr": "3理性指数",
				"updateBy": "13298598527",
				"updateTime": 1673855700000,
				"waveformRank": "13"
			},
			{
				"advise": "",
				"createBy": "13298598527",
				"createTime": 1673509746000,
				"description": "反映人的兴奋程度，指数越高人越兴奋。",
				"id": 83,
				"knowledge": "",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 4,
				"status": 0,
				"type": "exi",
				"typeStr": "4兴奋指数",
				"updateBy": "13298598527",
				"updateTime": 1673875397000,
				"waveformRank": "13"
			},
			{
				"advise": "",
				"createBy": "13298598527",
				"createTime": 1673509764000,
				"description": "反映人的抑郁程度，指数越高人越的抑郁程度越高。",
				"id": 84,
				"knowledge": "什么是抑郁抑郁是人的一种正常的负性情绪，是在人经历不好事情时候的一种情感体验。一般表现为：1、心境低落，主要表现为持久而显著的情感低落，抑郁，悲观，轻者闷闷不乐，无愉快感，兴趣减退；重者痛不欲生，悲观绝望，甚至有自杀的倾向或者自杀行为。2、思维迟钝，表现为思维联想速度缓慢，反应迟钝，思路闭塞。3、意志活动减退，呈持久显著的抑制，临床表现为行为缓慢，生活被动、懒散，不想做事，不愿意和周围人接触、交往，常常独自坐在一旁或整日卧床，闭门独居，回避社交；严重时，连吃、喝的生理需要和个人卫生都不顾，不修边幅。4、认知活动损害，对于近期事物记忆力下降，注意力障碍，反应时间延长。5、躯体症状，主要表现为睡眠障碍、乏力、食欲减退、体重下降、便秘，身体任何部分都可以出现疼痛、性欲减退、闭经等等。   一般情况下，人们在经历不好的事情之后，可能会出现这种抑郁的情绪，但一般不超过一周的时间。因为人们都有自我调节的能力，几天的时间可能抑郁的情绪就会明显的减轻甚至消失。如果超过两周以上的时间仍然没有明显的缓解，建议去医院进行诊治。",
				"level": 0,
				"levelStr": "无",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 5,
				"status": 0,
				"type": "yyzs",
				"typeStr": "5抑郁程度可能性",
				"updateBy": "13298598527",
				"updateTime": 1673855803000,
				"waveformRank": "13"
			},
			{
				"advise": "",
				"createBy": "13298598527",
				"createTime": 1673509778000,
				"description": "反映人被感觉或情绪等左右的程度，与理性相反，指数越高人越感性。",
				"id": 85,
				"knowledge": "",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 6,
				"status": 0,
				"type": "gxzs",
				"typeStr": "6感性指数",
				"updateBy": "13298598527",
				"updateTime": 1673875397000,
				"waveformRank": "13"
			}
		],
		"evaluateWaveformUnscrambles": [
			{
				"createBy": "admin",
				"createTime": 1688799348000,
				"id": 516,
				"label": "检测结果",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 1,
				"status": 0,
				"type": "consequence",
				"typeStr": "检测结果",
				"unscramble": "根据测评数据来看，您具备出色的组织管理能力和感统协调能力，同时也表现出了优秀的逻辑思维和分析能力。然而，您的精力似乎有比较大的消耗，可能是由于睡眠问题导致的。为了发挥您的潜力，建议您提高睡眠质量，以便更好地发挥您的能力和技能。此外，尽管您很努力，但在近期似乎出现了一些成就感不足和压力感较大的情况。您的兴奋度和幸福指数也有些偏低，这可能与您的睡眠和安排有关。为了增强工作和生活的乐趣，建议您调整关注点和时间安排，尝试拓宽社交关系并从中获得更多内在力量。让每一天都过得快乐，实现更高水平的表现和获得满满的幸福感。",
				"updateBy": "18942117941",
				"updateTime": 1688799348000,
				"waveformRank": "13",
				"δMaxLimit": 30,
				"δMinLimit": 20
			},
			{
				"createBy": "admin",
				"createTime": 1688799348000,
				"id": 515,
				"label": "对应建议",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 2,
				"status": 0,
				"type": "suggest",
				"typeStr": "对应建议",
				"unscramble": "从本次检测的数据来看，我们为您提供如下建议：首先，增加身体锻炼的次数和强度，这对于保持身体健康非常重要。如果可以的话，中午可以安排半小时左右的午休时间，或者通过冥想来放松身心。其次，我们需要调整好自己的心态，保持积极乐观的心态，这有助于改善睡眠质量。接着，我们要抓重要的事情，把握好“大局”，不能被琐事打乱了我们的节奏。同时，我们也需要放下束缚自己的想法，勇敢突破自己的个人局限，追求更高的目标。最后，我们需要放松自己，找到自己的乐趣，享受生活，更好的对待自己，我们要更好的照顾自己，提升身心健康，让每一天开心快乐。",
				"updateBy": "18942117941",
				"updateTime": 1688799348000,
				"waveformRank": "13",
				"δMaxLimit": 30,
				"δMinLimit": 20
			},
			{
				"createBy": "admin",
				"createTime": 1688799348000,
				"id": 514,
				"label": "解决方案",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 3,
				"status": 0,
				"type": "solution",
				"typeStr": "解决方案",
				"unscramble": "详细方案请咨询测评解读师！",
				"updateBy": "18942117941",
				"updateTime": 1688799348000,
				"waveformRank": "13",
				"δMaxLimit": 30,
				"δMinLimit": 20
			}
		],
		"foreword": "EEG脑波生理信号：医疗科研级精准率达≥95% 脑电生物信号测评技术：是强脑公司（Brain Co）科研创新与具影响力的技术项目 （脑电生物反馈技术、脑电生物测评技术）之一，由权威测机构报告验证：EEG-单 极导联精准率达≥95%，是医疗科研级的技术前沿地位…… 双域双值-项目测评技术：根据脑波振荡频带动率（PSD) 主要是通过非入侵式--脑电生物技术，使用采集脑电生理信号，对额叶、顶叶、枕 叶、颞叶及大脑皮层的网络系统建立分析模型与科学算法，技术，解读当前（测评 前3、12、24小时乃至28天周期内）情绪、心理特征、中枢神经、植物神经系统、 内分泌等与体质免疫功能有关所发生的细胞生物电与脑电信号反应…… 2亿研发成本·70名科学家·5年时间中检集团权威测试报告",
		"getEegLog": {
			"a": 22.7,
			"d": 40.2,
			"g": 29.5,
			"h": 23.5,
			"l": 29,
			"t": 34,
			"timestrapStr": "null"
		},
		"scientificPrinciple": "哈佛大学创新实验室、哈佛大学脑科学中心 2015年BrainCo成功诞生于哈佛大学创新实验室，独立开发脑机接口设备。同年BrainCo采用一套 全新的自学习理论--“基于机器学习归类下的人类学习&quot;理论，成功研发出当时市场上精度最高， 加密型最好的可穿戴脑电芯片，以及可以如意操控的机械手等。 </p> 脑科学，狭义地讲就是神经科学，是为了了解神经系统内分子水平、细胞水平、细胞间的变化过程， 以及这些过程在中枢功能控制系统内的整合作用而进行的研究。(美国神经科学学会)广义的定义是 研究脑的结构和功能的科学，还包括认知神经科学等。 EEG脑电图应用 人们情绪、思想、行为形成的本质是大脑神经网络之间相互交流，大脑与身体大量神经元的生物电 “同步&quot;脉冲作用，产生了脑电波’,人体的神经系统(包括中枢神经、植物神经)和免疫系统产生交 互作用后，它们随即开始寻找生理上的连结并产生免疫应答。 脑波与健康应用 认知科学在脑机接口领域，有以BrainCo为代表的非侵入式和以 马斯克的Neurolink为代表的侵入式进行了认知神经科学的应用 研究，包括:知觉功能(视听嗅味触)，认知功能(记忆、学习、执行 等等)，社会情绪功能，心理抑制等，是认知神经科学测量方法， 已促进、推动科学发展与社会效应。其中作为有潜力的非侵入式 脑机接口技术有良好的时间分辨率、易用性、便捷性。PSD脑波情绪检测科学原理",
		"userInfo": {
			"userAge": 23,
			"userBirthday": 946656300000,
			"userHeight": 185,
			"userName": "赵四",
			"userSex": 0,
			"userWeight": 150
		}
	},
` + mockDataTail

				//fmt.Println("mockData: ", mockData)
				// 创建一个新的请求实例
				req, err := http.NewRequest("POST", "https://127.0.0.1:8089/MentalClinic/data", bytes.NewBuffer([]byte(mockData)))
				if err != nil {
					fmt.Println("Error creating request:", err)
					return
				}
				// 设置请求的Content-Type为application/json，这是大多数服务期望的格式
				req.Header.Set("Content-Type", "application/json")

				// 发起请求
				//client := &http.Client{}
				// 创建一个客户端，并配置自定义的Transport
				client := &http.Client{Transport: tr}

				resp, err := client.Do(req)
				if err != nil {
					fmt.Println("Error making request:", err)
					return
				}
				defer resp.Body.Close()

				// 读取响应体
				body, err := ioutil.ReadAll(resp.Body)
				if err != nil {
					fmt.Println("Error reading response:", err)
					return
				}

				// 打印响应状态和响应体
				fmt.Println("Response status:", resp.Status)
				fmt.Println("Response body:", string(body))

			}(UUID)

			for i := 0; i < 20; i++ { // 循环20次
				template := AppCache[UUID].UserInfoTemplate
				dpId := AppCache[UUID].DpId
				uid := AppCache[UUID].UserId

				if template != "" {

					/*
					  模拟用户登录
					*/
					//用户已经存在，模拟用户登录
					session := sessions.Default(c)
					//在这里修改,3表示有效时间为1800秒
					session.Options(sessions.Options{MaxAge: 1800})
					// 设置用户ID到会话中
					session.Set("UserID", utils.Strval(uid))
					if err := session.Save(); err != nil {
						// 如果保存Session失败，处理错误
						c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": "模拟登录失败"})
						return
					}
					var userId = session.Get("UserID").(string)
					fmt.Println(config.ColorPurple, "从session中获取userId: ", userId, config.ColorReset)
					config.WorldLoginCacheCode.Set(userId, true, time.Hour)

					// 构建响应数据
					response := gin.H{
						"code":           1,
						"callBackStatus": "获取数据成功!",
						"UserId":         uid,
						"UUID":           UUID,
						"DpId":           dpId,
					}

					// 序列化响应数据为 JSON 字符串
					jsonData, err := json.Marshal(response)
					if err != nil {
						// 处理序列化错误
						fmt.Println("序列化 JSON 数据出错:", err)
						return
					}
					// 打印 JSON 数据到控制台
					fmt.Println(string(jsonData))
					// 数据已到达，返回成功响应
					c.JSON(http.StatusOK, response)

					fmt.Println(config.ColorPurple, "取出缓存的UserId: ", uid, "取出缓存的dpId: ", dpId, config.ColorReset)
					return
				}

				time.Sleep(1 * time.Second) // 等待1秒钟后再次检查数据是否到达
			}
			// 如果这个点还没有数据，说明请求超时
			//c.JSON(http.StatusOK, gin.H{"code": 0, "message": "请求超时, 脑机接口数据未到"})
			//return

		} else if code == 500 || resp.StatusCode == 500 {
			c.JSON(http.StatusOK, gin.H{"code": 0, "message": "请求失败，请求ID已存在"})
		}

	}
	/*
	  如果前端传的是学号和学校名称则执行下面的操作
	*/
	SerialNo := c.PostForm("SerialNo")
	SchoolName := c.PostForm("SchoolName")
	if SerialNo != "" && SchoolName != "" {
		//将学号添加至缓存
		AppCache[UUID].SerialNo = SerialNo
		AppCache[UUID].Unit = SchoolName

		requestBody := RequestBody{
			SerialNo:        SerialNo,
			SchoolName:      SchoolName,
			CallbackAddress: backUrl,
			RequestNo:       UUID,
		}
		// 将结构体实例转换为 JSON 格式的字节流
		jsonData, err := json.Marshal(requestBody)
		if err != nil {
			fmt.Println("Error marshalling JSON:", err)
			return
		}
		url := config.Conf.MentalClinicUrl
		// 创建请求
		req, err := http.NewRequest("POST", url, bytes.NewBuffer(jsonData))
		if err != nil {
			c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
			return
		}
		// 添加请求头
		for key, value := range headers {
			req.Header.Set(key, value)
		}
		// 发送请求
		client := &http.Client{}
		resp, err := client.Do(req)
		if err != nil {
			c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
			return
		}
		defer resp.Body.Close()

		// 读取响应体
		body, err := ioutil.ReadAll(resp.Body)
		if err != nil {
			c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
			return
		}
		var result map[string]interface{}
		err = json.Unmarshal(body, &result)
		if err != nil {
			c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
			return
		}
		code := int(result["code"].(float64))

		// 将响应体作为字节切片发送回客户端

		if code == 200 || resp.StatusCode == 200 {
			fmt.Println("响应体: ", body)

			// 异步执行ProcessData逻辑
			go func(uuid string) {
				// 延迟1秒以模拟异步行为
				time.Sleep(1 * time.Second)

				// 创建跳过证书验证的Transport
				tr := &http.Transport{
					TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
				}

				// 准备模拟数据
				// 使用fmt.Sprintf拼接字符串，其中包含uuid
				mockDataTail := fmt.Sprintf(`"requestNo": "%s"}`, UUID)
				mockData := `{
	"baseInfos": {
		"evaluateWaveformAdvantages": [
			{
				"createBy": "13298598527",
				"createTime": 1673423384000,
				"description": "对自身能力的信心。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/8087b86752b444a29acc1e54948d469c.png",
				"id": 164,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 1,
				"status": 0,
				"type": "poc",
				"typeStr": "1自信力",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423408000,
				"description": "* 无论遇到什么困难，都能坚持不懈的完成任务，在任何困难条件下都能长时间保持高水平投入状态。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/7c8bb5df94964d258f542aec895b303a.png",
				"id": 165,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 2,
				"status": 0,
				"type": "wp",
				"typeStr": "2毅力",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423434000,
				"description": "* 即自我控制的能力，指一个人对自身的冲动、感情、欲望施加的正确控制。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/a7733def606b4677a54b3a22cafd8548.png",
				"id": 166,
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 3,
				"status": 0,
				"type": "honest",
				"typeStr": "3自控力",
				"updateBy": "13298598527",
				"updateTime": 1673875267000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423464000,
				"description": "* 勇于承担责任。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/5e77e8e89e7c4f1f9c941568ed912ccc.png",
				"id": 167,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 4,
				"status": 0,
				"type": "sor",
				"typeStr": "4责任感",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423504000,
				"description": "* 在压力下，在行动的最后期限之前及时做出决策。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/19356f9cb7784dcd8141aa07dde9222f.png",
				"id": 168,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 5,
				"status": 0,
				"type": "at",
				"typeStr": "5果断性",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423532000,
				"description": "* 评估问题解决情境下的风险，可能的结果以及可能造成的反响、仔细权衡行动方案，在行动上要有耐心。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/33361823a8ed44559acac1de87c30aa3.png",
				"id": 169,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 6,
				"status": 0,
				"type": "aoj",
				"typeStr": "6判断能力",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423569000,
				"description": "* 利用手中资源创造性的解决问题。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/0237a5795b9e4dc0811f64f76654ba5f.png",
				"id": 170,
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 7,
				"status": 0,
				"type": "ps",
				"typeStr": "7问题解决",
				"updateBy": "13298598527",
				"updateTime": 1673875267000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423597000,
				"description": "* 有效的沟通，与他人沟通良好。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/5d75b2f9f98e4a5b98391213ec49c5ed.png",
				"id": 171,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 8,
				"status": 0,
				"type": "atc",
				"typeStr": "8沟通能力",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423623000,
				"description": "* 情绪富有弹性，能忍受困难和挫折，在紧急时刻或者压力之下依然保持战斗力。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/9a0817c2479645efa51e4f9846ff3132.png",
				"id": 172,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 9,
				"status": 0,
				"type": "apa",
				"typeStr": "9抗压能力",
				"updateBy": "13298598527",
				"updateTime": 1673875279000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423644000,
				"description": "* 自我约束与自我调节能力。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/b58b3cf4e5c8402287091c6c6f1fc068.png",
				"id": 173,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 10,
				"status": 0,
				"type": "sd",
				"typeStr": "10自律",
				"updateBy": "13298598527",
				"updateTime": 1673875280000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423671000,
				"description": "* 将群体目标至于个人目标之上，荣辱与共。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/c9d1f69a6a954a8f8c8f58f501783d45.png",
				"id": 174,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 11,
				"status": 0,
				"type": "cpa",
				"typeStr": "11合作",
				"updateBy": "13298598527",
				"updateTime": 1673875280000,
				"waveformRank": "13"
			},
			{
				"createBy": "13298598527",
				"createTime": 1673423698000,
				"description": "* 立即改变需求的行动，修改计划以适应形势。",
				"icon": "https://ea-focus.oss-accelerate.aliyuncs.com/ea-focus-admin/upload/2023/01/11/68cf394014ea4c9d8194c4b7ecf14c9f.png",
				"id": 175,
				"level": 3,
				"levelStr": "强",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 12,
				"status": 0,
				"type": "apb",
				"typeStr": "12适应力",
				"updateBy": "13298598527",
				"updateTime": 1673875280000,
				"waveformRank": "13"
			}
		],
		"evaluateWaveformCapacitys": [
			{
				"advise": "如何提高共情能力？ 1.学会换位思考：能从对方角度为对方的行为寻找合理性，以最大限度地理解对方。  2.学会倾听：指能全身心地聆听对方的表达。倾听不仅指听取其口语表达的内容，还包括观察非语言的行为，如动作、表情、声音语音语调等。  3.表达尊重：尊重对方的个性及能力而不是凭自己的感情用事。接纳对方的信念和所做出的选择或决定，而不是评论或试图替其做决定。",
				"createBy": "13298598527",
				"createTime": 1673505354000,
				"description": "共情能力是一种将自己置于他人的位置、并能够理解或感受他人在其处境里所经历的事物的能力。",
				"id": 145,
				"knowledge": "什么是共情能力指能够站在对方立场设身处地思考、于人际交往过程中能够体会他人的情绪和想法、理解他人的立场和感受，并站在他人的角度思考和处理问题。它主要体现在情绪自控、换位思考、倾听能力以及表达尊重等与情商相关的方面。共情之所以是利他行为的基础，是因为具备共情特质的人能切身感受到别人的需要与苦恼，并能在必要时以得体和尊重的方式向他人提供支持与帮助，因此，非常有助于健康人际关系的建立。日常生活中，有共情能力的人较少与别人发生冲突，因为他（她）总能最大限度地理解别人，并以平和的心态与人相处。共情（同理心）与同情心的区别同情心，是认知到别人的痛苦，从而引起恻隐之心。共情能力是能够感同身受，设身处地为他人着想，实际感受到他人的痛苦。当听到他人遭遇不幸时，有同情心的人会说：“我真的替你感到难过。”但是有共情能力的人却会这样说：“我也遇到过这样的事，我知道这是什么样的感觉。”同理心是牵涉两个人的互动过程，必须认真地倾听，而且了解、接纳对方的感觉，这是一种认同他人体验的态度。同理心是一种关心与爱的艺术，是高情商的表现。共情能力的影响因素遗传：有研究表明，遗传在共情中的影响可能达到30%以上。性别：多项研究证实，女性共情能力普遍要高于男性。",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 1,
				"status": 0,
				"type": "ec",
				"typeStr": "1共情能力",
				"updateBy": "13298598527",
				"updateTime": 1673875339000,
				"waveformRank": "13"
			},
			{
				"advise": "攻击性行为在儿童时期的表现   孩子在成长过程中会逐渐学习控制和表达自己的情绪和行为、以积极的方式与他人沟通。在学会之前，孩子可能会试图用攻击性行为（打、踢、咬、抓等）来表达自己。攻击性行为的原因有很多，如表达需要关注、快速达到自己的目的、在行为中获得力量感等。",
				"createBy": "13298598527",
				"createTime": 1673505376000,
				"description": "暴力行为，即攻击性行为，指身体、语言或心理上导致人或物受到伤害的行为（如打、踢、咬、抓、威胁、羞辱、责备、破坏等）。抗暴力行为能力指的是阻止或限制攻击性行为的能力。",
				"id": 146,
				"knowledge": "攻击性行为在儿童时期的表现   孩子在成长过程中会逐渐学习控制和表达自己的情绪和行为、以积极的方式与他人沟通。在学会之前，孩子可能会试图用攻击性行为（打、踢、咬、抓等）来表达自己。攻击性行为的原因有很多，如表达需要关注、快速达到自己的目的、在行为中获得力量感等。影响攻击性行为的因素 1.生物因素：男性比女性具有更多的身体攻击性行为，而女性则具有更多非身体攻击性行为（如言语攻击）。除了性激素之外，肾上腺皮质激素和垂体促肾上腺皮质激素对攻击行为也有调节作用。 2.疾病因素：癫痫、痴呆症、自闭症、注意力缺陷多动障碍、精神分裂症和脑损伤等均可导致攻击性行为增加。 3.其他因素：药物酒精滥用、不健康的家庭关系、童年目击进攻性行为和慢性压力等均可导致攻击性行为增加。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 2,
				"status": 0,
				"type": "rtv",
				"typeStr": "2抗暴力行为能力",
				"updateBy": "13298598527",
				"updateTime": 1673855161000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提高情感表达能力？ 1.通过写日记、读故事、看电影、接触艺术作品等方式，增加自己与情绪接触的机会。 2.使用语言、文字等方式表达自己的内心感受。 3.多倾听、换位思考，提高对他人的理解，表达对他人的关注。 4.若感觉自己受述情障碍影响较大，可以寻求心理咨询师的帮助。",
				"createBy": "13298598527",
				"createTime": 1673505395000,
				"description": "情感表达能力弱的人格特质也被称为述情障碍，表现为缺乏情感意识、难以识别和表述感受和情绪。",
				"id": 147,
				"knowledge": "述情障碍有什么表现？述情障碍，即情感表达能力弱，表现为对于情绪的识别不敏感，可能会看起来比较冷漠、有距离感。有述情障碍特质的人，可能会对积极的情绪（如愉悦、惊喜）无动于衷，也可能会面对消极情绪（如紧张、焦虑）异常平静。另一方面，虽然大脑处理情绪的能力较弱，身体的感官却不会消失甚至会变得更敏感，如在面对压力的时候可能会肩膀僵硬。影响述情障碍的因素 1.社会文化差异：不同文化背景下的人会表现出不同水平的述情障碍，如亚裔美国人的述情障碍水平高于欧裔美国人。 2.社会经济地位：受教育程度低和贫困的人更容易出现高述情障碍，因为自身的社会经济状况要求他们不得不为了生存而努力，不鼓励他们情绪和情感的加工和表达。 3.家庭环境：不良的家庭环境，可能会影响儿童早期社会化、压抑对依恋的感受意识，是述情障碍发展的危险因素。与述情障碍相关的疾病精神疾病（孤独症、抑郁症、精神分裂症等）患者大多存在述情障碍。糖尿病、冠心病、高血压、脑卒中、银屑病、哮喘、十二指肠溃疡、乳腺癌等疾病患者较普通人群受述情障碍影响比例更高。",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 3,
				"status": 0,
				"type": "aea",
				"typeStr": "3情感表达能力",
				"updateBy": "13298598527",
				"updateTime": 1674098238000,
				"waveformRank": "13"
			},
			{
				"advise": "如何克服社交恐惧？ 1.找出原因尝试克服：找出具体触发你恐惧或者焦虑的因素，以及你害怕的场景。根据你分析的触发因素和惧怕场景，通过了解什么是导致你焦虑的反应，可以以更积极或有针对性的方式来处理这些焦虑情绪。 2.尝试让自己平静的方法：学会识别自己对焦虑或者恐惧的身体反应，然后尝试那些让自己平静下来的方法：比如深呼吸，或者将身体收紧三秒再放松，听一些能让你心情愉悦的歌，或者静下心来冥想等。 3.改善自己的想法：用积极的想法来代替那些大脑自动产生的消极想法，培养更多的自信。 4.减少对自己的注意力：观察你周围的人和环境，将专注点放在别人正在说的话上，而不是过多专注于自己。 5.尝试多与理解你的人沟通：试着去寻找那些与你有同样烦恼，理解你的人，并与他们多交流。也许可以帮助你们互相改善，或者让你积累一些自信。他们的一些成功经历也能成为你借鉴的经验。",
				"createBy": "13298598527",
				"createTime": 1673505427000,
				"description": "抗社交恐惧能力是指抵抗因存在与社交场合而产生焦虑、惊恐的能力。",
				"id": 148,
				"knowledge": "什么是社交恐惧社交恐惧是一种对任何社交或公开场合感到强烈恐惧或忧虑的心理状态。社交恐惧的表现具有社交恐惧症的人在陌生人面前或可能被别人仔细观察的社交或表演场合，有一种显著且持久的恐惧，害怕自己的行为或紧张的表现会引起羞辱或难堪。甚至有些人对参加聚会、打电话、到商店购物、或询问权威人士都感到困难。产生社交恐惧的原因 1.生理原因：有研究表明可能是由于人体内被称为“5-羟色胺”的化学物质失调所致，这种物质负责向大脑神经细胞传递信息，其过多或过少都可能引起人们的恐惧情绪。 2.心理原因：社交恐惧症患者一般自尊心较强，害怕被别人拒绝，或者对自己的外貌没有信心。 3.家庭原因：从小性格受到压抑，或者是父母没有教会他们社交的技能，或者是家庭搬迁过于频繁。 4.社会原因：本身所处的社会环境较为恶劣，与人交往时受到的挫折居多。 5.思维方式：不正确的思维方式造就了社交恐惧症，比如过分的完美主义者受到打击后带来的过分自身反省，每个社交恐惧症患者都应该对“自己”有充分的了解。 6.遗传因素：研究发现恐惧症具有家族遗传倾向，尤其影响到女性亲属。",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 4,
				"status": 0,
				"type": "ai",
				"typeStr": "4抗社交恐能力",
				"updateBy": "13298598527",
				"updateTime": 1673875339000,
				"waveformRank": "13"
			},
			{
				"advise": "如何降低孤独感？ 1.走出家门，感受室外的空气和阳光，和周围的人打个招呼。 2.做些锻炼，慢跑、游泳、瑜伽、拳击都是不错的选择。 3.给家人或朋友打个电话、闲聊一会，约个时间出来见面。 4.控制手机、电脑、电视等屏幕使用时间。 5.试着养一个宠物。 6.规律作息，睡个好觉。 7.了解自身，知道孤独不是永久的状态。 8.如果以上方法都不奏效，寻求心理医生或咨询师的帮助。",
				"createBy": "13298598527",
				"createTime": 1673505444000,
				"description": "孤独指的是社交需求没能得到满足时的一种心理感受。抗孤独能力强的人更不容易感到孤独。",
				"id": 149,
				"knowledge": "经常感到孤独正常吗？人人都会感到孤独，性别、年龄、样貌、社会经济条件等都不能让你免于孤独，对此不用过分担心。孤独与独处状态不能完全等同，孤独可能源于长时间的独处，或者仅仅是感到不被他人理解。孤独可能造成的影响孤独会对心理和身体造成一系列的影响：孤独让人更容易感到抑郁、睡眠质量下降、执行功能受损、认知能力下降、免疫力下降、心血管功能下降；孤独是阿兹海默病、心脏病、2型糖尿病、关节炎等疾病的风险因素；孤独会提高“战斗或逃跑反应”（fight-or-flightresponse，当人面对威胁时的一系列生理反应）；孤独还会影响人的寿命。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 5,
				"status": 0,
				"type": "ala",
				"typeStr": "5抗孤独能力",
				"updateBy": "13298598527",
				"updateTime": 1673854765000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提升积极情感？   记录你的积极情感：记录下你在生活中已经经历且熟悉的积极情感。当你发现有新的积极情感时，首先注意，直到你能确认时再记录下来。当记录到一定数量时，看着名单，回忆每一个情感产生时的活动、情景或人物。 关注一种积极情感并且加强它：回忆该积极情感产生时的情况，尝试将这些场景增加到日常生活中，以此加强该类型的积极情感。创造积极情感的宝箱：有时候我们会忘记曾经积极情感的来源，因此积极情感的宝箱变得很有意义。可以将那些产生积极情感的物品保存下来，比如照片、纪念品、日记本、卡片等。将它们收藏在你的宝箱里，你可以随时增加、减少、整理它们。当你开心或不开心的时候，将宝箱打开，让它们增加你的积极情感。",
				"createBy": "13298598527",
				"createTime": 1673505460000,
				"description": "积极情感，又叫积极情绪或正性情绪，是由于体内外刺激、事件满足个体需要而产生的伴有愉悦感受的情绪。",
				"id": 150,
				"knowledge": "对于积极情感的描述罗素：“积极情感就是当事情进展顺利时，你想微笑时产生的那种好的感受”。 Fredrickson：“积极情感是对个人有意义的事情的独特即时反应，是一种暂时的愉悦”。孟昭兰：“积极情感是与某种需要的满足相联系，通常伴随愉悦的主观体验，并能提高人的积极性和活动能力”。十种人们最常感受到的积极情感形式 1喜悦（Joy）当我们感受到喜悦时，我们一般处于这样的情形下：一切按照预定的方式发展，结果符合我们的期待，甚至比我们所期待的要更好。喜悦是一种轻快而明亮的感觉，当我们感到喜悦，我们会感到浑身轻松，甚至周围的事物看起来也更生动、顺眼，我们可能会想加入他人的谈话、对接下来的事跃跃欲试。 2感激（Gratitude）当我们意识到他人对我们的付出，我们会体验到感激。有时我们感激的对象也不一定是人，也可能是感受到某种事物带给我们益处。当我们赞赏人、事、物的可贵，感激就出现了。感激会带给我们“想要付出回报”的冲动，我们会希望对帮助过我们的人做点好事，也可能会想通过帮助其他人来把自己受过的恩惠传递出去。 3宁静（Serenity）宁静是一种绵柔、低调、放松版本的喜悦，通常发生在感觉深处安全而美好的环境中。宁静会让人们更加需要沉浸在当下，去品味当前的感受。 4兴趣（Interesting）兴趣是我们在安全的环境中，被一些新颖的人、事、物吸引了注意力时感受到的情绪。我们会被兴趣牵引着，去探索、尝试，去消除神秘、了解更多。 5希望（Hope）相比平淡的日常，往往在事情发展对我们不利或者存在不确定性时，我们更容易感受到希望。希望的核心是我们相信事情能好转、好事有可能发生的信念和愿望。因为相信未来还有可能是好的，我们才愿意继续生存下去。 6自豪（Pride）自豪是一种因为我们是、我们做了、或我们达到了什么而产生的满足感。这是一种很重要的感情，因为它提高了我们的自尊，让我们对自己更有信心，并且使我们面对负面情绪时更加坚强。 7逗趣（Amusement）生活充满了幽默的故事或玩笑，并且那些有趣的经历会让你开怀大笑，真正的感受到被逗趣了。 8激励（Inspiration）当我们看到完美的杰作、或者听到伟大的演讲，会因此而产生被激励的感觉。它是一种可以改变心情的非常强烈的驱动力量，让人去追逐完美。 9敬畏（Awe）是一种面对崇高、狞厉、神秘事物所体验到的惊奇、畏惧却又爱戴、崇敬的感受。在对自然、社会、艺术的审美中，都可能产生。 10爱（Love）是最为复杂的情感。当我们还是刚出生的婴儿时，第一体验到如此广泛而又强烈的情感。最纯净的形式是无条件的爱，只有付出而不求回报。积极情感的影响：积极情感带来开放性，提升我们的创造力。积极情感带来积极的人际互动体验。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 6,
				"status": 0,
				"type": "pac",
				"typeStr": "6积极情感能力",
				"updateBy": "13298598527",
				"updateTime": 1673854980000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提高执行能力？有研究表明，通过专业的针对性行为训练可以提高执行能力，包括任务转换训练、记忆更新训练、反应抑制训练等。积极参加有氧运动。参与有氧运动可以有效改进大脑前额皮质功能；同时，人体在有氧运动过程中会分泌内啡肽，可以增加愉悦程度，多人参与的有氧运动还能增加社会联结，而悲伤、孤独等负面情绪会带来执行能力的降低。",
				"createBy": "13298598527",
				"createTime": 1673505474000,
				"description": "执行能力是指完成特定的目标或者任务时，需要动用的一些高级的认知功能和过程，如专注力、工作记忆力、控制力、认知灵活性等。其本质是对认知过程进行控制和调节的能力，一般也称作执行功能。",
				"id": 151,
				"knowledge": "什么是执行能力  执行能力，或者叫执行功能（Executive function），是人们在执行任务，完成具体的目标时需要的有意识的心理控制能力，是一种非常综合、高级的认知功能。执行能力主要体现在以下几个方面：计划和决策能力：能分析需要完成任务需要的步骤，并且构建完整的方案。管理能力：能对时间、资源进行合理的安排。工作记忆能力：在完成复杂的工作的时候，能够记住其中关键的信息，并且利用这些信息制定解决问题的计划。控制力：能在行动时抑制本能的冲动，以完成任务为目标，不拖延，在完成的过程中也能控制自己的行为，管理自己的情绪，并且不被其他事物分散注意力。适应能力：在面对新的变化、阻碍、发生错误的时候能及时修改计划，对环境的变化能有足够的适应。执行能力的发展过程执行能力是成熟的最晚的心理功能之一，在人生的各个阶段都处于不断发展和变化中。 幼儿期：注意力、工作记忆力等能力在幼儿时期就会出现。在幼儿园时期（3～5岁），适应能力、计划性等能力也会开始快速形成。当然，在这个时期，还不具备成熟完整的执行能力，所以在幼儿期经常会犯错误，这不是能力不足导致的，而是由于大脑的决策能力、执行能力在这个时期还不够成熟。青春期前：这个时期，执行能力会再有一次快速提升，尤其是语言工作记忆力，有目的的行为、注意力、控制力等能力有显著提升。不过和幼儿期相似，整体的执行能力还处于比较有限的状态。青春期：青春期之前的出现的能力在这个阶段继续保持增长，并且系统的整合起来。专注能力可能在这个阶段会有一个爆发式增长。成年期：成年后的最重要的变化就是前额叶皮层的持续髓鞘化。20-29岁是执行能力最强的一个时期，使得大多数人在这个时期可以参与非常具有挑战的工作。年龄逐渐增大后，执行能力也会慢慢有相应的下降。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 7,
				"status": 0,
				"type": "ate",
				"typeStr": "7执行能力",
				"updateBy": "13298598527",
				"updateTime": 1673854790000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提高抗职业倦怠能力？ 1.通过积极的沟通和反馈，获得同事或上级的情感支持，调整来自工作中的各类压力源。 2.平衡工作和生活时间，保证充足的睡眠。",
				"createBy": "13298598527",
				"createTime": 1673505511000,
				"description": "抗职业倦怠能力是指当工作压力大等职业因素出现时，可以正确克服身体、情绪或精神疲惫状态的能力。",
				"id": 152,
				"knowledge": "",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 8,
				"status": 0,
				"type": "rtb",
				"typeStr": "8抗职业倦怠能力",
				"updateBy": "13298598527",
				"updateTime": 1673855019000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提高主观幸福感？ 1.维护良好的人际关系：幸福的人更可能被别人选做朋友和信任的对象，更多的去帮助别人，减少消极情绪，建立互信关系满足人的归属需要，会令人感到幸福和满意。 2.创造满意的居住环境：我们要选择人身安全、让自己和家人舒适，有优美自然风光、气候舒适、可以选择能欣赏到音乐和艺术的地方居住。 3.积极增加体育锻炼：锻炼的短期效果导致大脑产生内啡肽和类吗啡的释放；而长期有规律的锻炼减少了抑郁和焦虑，提高了我们工作的准确性和速度，提升了我们的自我概念，促进了心血管的健康和机能。另外，有规律的锻炼的人经常和别人结伴运动，也能在这种额外的社会支持中受益，因此我们应该保持良好的健康状况，定期参加体育锻炼。 4.适度的业余活动：有研究表明，参加休闲或运动组织，尤其是那些包含舞蹈、音乐、义工或运动组织，可以带来更高的主观幸福感。",
				"createBy": "13298598527",
				"createTime": 1673505535000,
				"description": "主观幸福感指个体根据自定的标准对其生活质量的整体性评价，它是个体对于自己是否幸福的一种主观感受。",
				"id": 153,
				"knowledge": "",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 9,
				"status": 0,
				"type": "swb",
				"typeStr": "9主观幸福感",
				"updateBy": "13298598527",
				"updateTime": 1673875339000,
				"waveformRank": "13"
			},
			{
				"advise": "如何克服情绪化进食？找到引发情绪化进食的原因：想要停止情绪化进食，就要正视它，剖析在什么诱因下自己开始进食，如果这一模式相对清晰，改正起来也相对容易。转移注意力：把心思从吃上暂时移开，想吃东西时，尝试让注意力分散10分钟，比如立刻起身去做别的事情（收拾屋子/去锻炼/洗个脸敷个面膜）或者跟朋友聊天打电话等，让吃的念头被分散。准备健康食物：在手边准备好健康的食物，比如即食燕麦粥、速溶番茄汤、无糖全麦面包等相对健康又有饱腹感的食物，当出于心理需要想吃东西的时候，拿健康的食物来满足对情绪化饮食的迫切需要。培养应对情绪问题的习惯：可以尝试找一些替代食物的方式，比如压力大的时候，去洗个热水澡或者去做按摩；感到抑郁孤独的话，试试养一只宠物或者定期与好友聚会或网上交流；或者把某项运动发展为自己的爱好，比如瑜伽。养成健康的生活习惯：早睡早起不熬夜，只有当睡眠质量高、身心放松，休息充分时，才能更好地处理生活中的困难以及应对多变的环境；规律且适度的运动，定期减压、放松，同时进行一些积极地社交活动，都有助于改善情绪化进食。",
				"createBy": "13298598527",
				"createTime": 1673505566000,
				"description": "抗情绪化暴食能力是指对抗由于情绪方面的原因而过度进食的能力。",
				"id": 154,
				"knowledge": "什么是情绪化进食情绪化进食是为了满足情绪而进食，而非因为生理性饥饿而吃。压力会导致人体皮质醇水平升高，而高碳水，高糖和高脂肪的食物可以降低压力荷尔蒙，在沮丧和不安的时候拼命吃高热量食物会减轻心理压力和负面情绪,然而食物入口的快感仅能维持1分钟,接下来是吃完之后懊悔、羞愧、自责的心理，肥胖也随之产生。因此，情绪化进食并不能解决人们实际生活中的问题和情绪,反而会加剧人们情绪的不稳定性，如果不加控制的话，情绪化进食有可能会演变成暴食症或神经性贪食症。你是一名情绪化进食者吗？当你感到紧张的时候是否吃的更多？当你并不感到饥饿或者已经饱足的时候是否进食？当你感到伤心、愤怒、无聊、焦躁时通是否过进食来抚慰自己、恢复平静，通过进食来让自己感觉更好？你是否用食物来奖励自己？你是否经常吃到过撑？你是否对食物感到无力或者不受控制？情绪化饥饿与生理化饥饿的不同情绪化饥饿突如其来：情绪化饥饿突如其来，它在一瞬间让你觉得不可抗拒而又十分迫切；生理性饥饿是逐渐而来，想吃的愿望并不十分迫切，也不会要求立即的满足（除非你已经很久没有吃东西了）。情绪化饥饿渴求特定的食物寻求安慰：情绪化饥饿渴求特定的食物寻求安慰；当你处于生理性饥饿时，几乎所有的食物听起来都很好吃——包括例如蔬菜的健康食品。情绪化饥饿往往导致暴食：当你意识到之前，你已经吃了一大袋薯片或者一整盒冰激凌，却并没有真正地注意到什么或者充分地享受食物。而当你因为生理性饥饿而进食时，往往更加能够意识到自己在做什么。情绪化饥饿并不会因为吃饱而得到满足：你只会想吃得越来越多，往往直到撑得很不舒服的时候才会停下来。另一方面，生理性饥饿并不会导致你吃撑，当你的胃填满的时候，你就感到了满足。情绪化饥饿往往导致后悔、内疚和羞愧：情绪化饥饿往往导致后悔，内疚和羞愧。当你因为生理性的饥饿去吃东西时，不太可能感到内疚和羞愧，因为你仅仅是给予身体所必需的东西。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 10,
				"status": 0,
				"type": "rteo",
				"typeStr": "10抗情绪化暴食能力",
				"updateBy": "13298598527",
				"updateTime": 1673855127000,
				"waveformRank": "13"
			},
			{
				"advise": "如何提高组织管理能力管理者的组织管理能力，除了体现在建立机构、团队建设之外，还充分体现在个人的综合素质与综合能力之上。个人组织管理能力是管理者学习能力、判断能力、决策能力、实施能力、协调能力等的集合，不是单一的某种能力的体现。所以，提高管理者的组织管理能力，必须从多方面努力，从多种能力的提高人手。(一)增强学习能力从书本上学习，从实践中学习，勤于思考。1、根据工作要求及个人情况，制订出学习内容、目标、进度等的学习计划。有所选择，有所规划，树立志向、志气、志趣，长期坚持，学有所为，有的放矢。2、善于利用时间、讲究学习方法。3、提高学习效率、学以致用、勤于思考、建立自己的体系；(二)增强判断能力，对事物有自己清晰明确的判断标准，思路清晰，能更快地作出准确无误的判断。提高判断能力既需要理论上加以训练，又需要在工作实践中不断提高。1、正确掌握运用各种类型的判断逻辑形式等必要知识；2、加强个人思想修养，不唯书、不唯上、有自信、敢创新，无私、无畏；3、努力参与社会实践，在实践中提升。4、不轻易判断他人，要客观判断自己，在管理工作中不能以貌取人，仅凭第一印象就对一个人下判断。要听其言，观其行。而对待自己，则要有自知之明，不能过于自信，自大狂妄。也不能过于悲观，缺乏自信。(三)增强决策能力，决策能力是组织管理能力的核心。1、提高决策素质是首要，决策者要作出正确的决策，必须不断提高自身的思想品质修养、知识能力修养、性格气质修养、情绪情感修养等。2、掌握决策原则，决策的基本原则有求实原则、系统原则、民主原则、可行原则、创新原则等。求实原则要求领导者决策符合客观实际；系统原则要求领导者决策必须在综合一切信息的基础上考虑各种情况并提出相应的解决方案；民主原则要求决策者必须按照科学、透明的程序在集体智慧的基础上作出决策；可行原则要求领导者决策是可以在实践中实施的；创新原则要求领导者决策必须以新的思路去解决新的问题。3、优化决策程序，领导者在决策时，不仅要按照一般程序来进行，而且要尽可能根据科学高效原则减少一切不合理环节，在决策过程中要注意优化每一个环节，在各种方案中进行优选。4、掌握科学的决策方法，决策是有方法可循的，按照不同的决策类型，应当选择不同的决策方法，常见的可分为定性决策和定量决策两大类。领导者可根据情况适当选择或综合使用，要在实践中体会各种决策方法的优缺点，对其中一些适合自身的决策方法能够运用自如。(四)增强协调能力，领导协调是组织管理工作中经常性的工作。包括协调与上级的关系、与下级的关系、与同级的关系，各种关系的协调不可有固定的模式，要因势利导，针对不同对象在方法上有所不同，有所选择。基本的方法有:1、疏通阻塞：各种不协调的现象的出现，大部分是因为信息阻塞引起的。领导者要善于疏通阻塞。重大问题先协商后再决策，一般问题先通气后再执行，暂时有分歧的就冷处理，善于等待，疏通关节2、多商量：商量是协调最重要的环节。领导者要用平等的态度对待别人，不要强加于人。既然是商量，无论对方和自己的关系如何，地位如何，都要把对方摆在平等的地位。3. 化解矛盾：领导者无时无处不生活在矛盾中，诸如利益冲突、心理冲突、角色冲突、职权冲突、目标冲突等。要正视矛盾，要善于化解矛盾。可采用以冷制热法、含糊处理法、春风化雨法、接受时间法、彼此退让法等。4、保持平衡：各种社会关系需要互相依赖，竞争与合作是一种平衡。领导者要从总体上把握平衡，在坚持原则的前提下追求平衡，从动态中寻求平衡。要团结所有团队成员形成一个相与配合的组织，形成和谐的人际关系。(五)增强执行能力，好的想法、规划、计划，只有付诸于良好的执行才能够产生好的结果；",
				"createBy": "13298598527",
				"createTime": 1673505584000,
				"description": "组织管理能力是指为了有效地实现目标，灵活地运用各种方法，把各种力量合理地组织和有效地协调起来，为了实现目标不断优化人、事、物等的能力。包括协调关系的能力和善于用人的能力等等。",
				"id": 155,
				"knowledge": "",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 11,
				"status": 0,
				"type": "atom",
				"typeStr": "11组织管理能力",
				"updateBy": "13298598527",
				"updateTime": 1673854813000,
				"waveformRank": "13"
			}
		],
		"evaluateWaveformEmotions": [
			{
				"advise": "缓解焦虑的建议1、人如果抗焦虑能力弱，容易出现焦虑情绪，建议学会正确处理焦虑情绪；2、除了遗传因素，这类人也更容易出现焦虑情绪：性格自卑个性悲观的人、抽烟酗酒的人、遭遇压力和创伤的人；3、适度的焦虑可以使我们获得动力去做事，过度的焦虑却会使我们情绪痛苦，还会带来很多身体不适；4、使用以下方法可改善焦虑状态：a在安静的地方进行深呼吸、松弛肌肉和想象积极的画面（如静谧的自然景观）；b补充L-酪氨酸和维生素B12有利于舒缓情绪；   c调整到健康的生活习惯；   * 如需使用膳食补充剂，或需要心理干预，建议您咨询权威的健康专家或临床医生。",
				"createBy": "13298598527",
				"createTime": 1673509657000,
				"description": "反映人的焦虑程度，指数越高，人越焦虑。",
				"id": 80,
				"knowledge": "焦虑是什么 焦虑，是包含了担心、紧张、恐惧、尴尬等情绪的一种情绪的统称，它同时还带来了一些躯体上的生理唤醒，比如心跳加快、呼吸急促、手心出汗、眩晕、肠胃不适等躯体反应。焦虑完全是坏事吗适度的焦虑可以使我们获得动力去做某事，对于焦虑和绩效的研究得出的倒U型曲线也证实这一点，中等水平的焦虑可以得到最高的绩效。然而，过度的焦虑会带来情绪上的痛苦，还会带来很多躯体上的不舒服，比如会经常觉得累、肌肉紧张、脖子酸疼，睡眠质量下降、幸福感降低等。总之，过度焦虑会让人的生活质量下降很多。焦虑的影响因素遗传：从遗传学上来说，同卵双胞胎，如果一人患病，另一人患病的几率也会增加。如果父母中有焦虑症，子女患病的概率会上升到50%~75%。性格：一些个性上的特质，如自卑、过于依赖别人、自责、悲观、容易被压力击垮等，都会使你更容易产生焦虑。抽烟、酗酒：有研究表明，约有30%的严重焦虑症患者与酗酒以及使用滥用物有关。此外，焦虑症患者对尼古丁上瘾的机率比正常人高出约2倍。环境：令人感到有压力的生活事件及失落感可能诱发焦虑，如丧偶、离婚、失业、财务危机、出现不良健康状况等。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 1,
				"status": 0,
				"type": "ani",
				"typeStr": "1焦虑指数",
				"updateBy": "13298598527",
				"updateTime": 1673855557000,
				"waveformRank": "13"
			},
			{
				"advise": "几个改善睡眠的建议：1、身心放松：保持平和的心态，平时避免情绪过度波动，避免精神紧张、焦虑、抑郁的情绪。睡觉前可以听舒缓的音乐，让全身心得到充分放松，避免情绪激动。也可以通过睡前泡脚来促进血液循环，从而达到放松的状态，更容易进入深度睡眠；2、合理饮食：入睡前避免饥饿或者饮食过饱，避免饮酒、咖啡、浓茶等刺激性强的饮料；3、良好作息习惯：尽量早睡早起，午睡时间不宜过长，建立良好的生物钟，容易进入深度睡眠；4、睡眠环境良好：睡眠时寝室光线宜暗，周围环境安静，温度和湿度适宜，温度在22℃-26℃，湿度在40％-60％为宜。5、在日常生活中可以多晒晒太阳，在白天时多做体力活动或者体育锻炼，可以让身体处于略微疲劳的状态，提高睡眠深度。6、也可以吃药食同源的物质，比如酸枣、大枣可以起到安神的作用，提高睡眠质量。",
				"createBy": "13298598527",
				"createTime": 1673509678000,
				"description": "反映人的睡眠质量情况，指数越高，睡眠质量越好。它和人的深度睡眠时间、睡眠深浅、多梦、晚睡等相关。",
				"id": 81,
				"knowledge": "什么是深度睡眠深度睡眠是人体入睡以后大脑不进行活动的深度休息，是人体睡得最熟、最香的阶段，此时人听不到外界的声音，也极难被别人唤醒。人的机体进入睡眠状态后是先进入浅睡眠，然后自然进入深睡眠，然后又是浅睡眠，然后再是深睡眠，深睡眠和浅睡眠来回交替直到醒来。深度睡眠被称作是“黄金睡眠”，约占整个睡眠时间的25％，有助于人身体和细胞的修复，恢复人的脑力、体力和精力；深度睡眠也是孩子身体长高、智力发育的重要时间。",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 2,
				"status": 0,
				"type": "sli",
				"typeStr": "2睡眠指数",
				"updateBy": "13298598527",
				"updateTime": 1673875397000,
				"waveformRank": "13"
			},
			{
				"advise": "",
				"createBy": "13298598527",
				"createTime": 1673509708000,
				"description": "反映处理问题不被情绪和感觉所左右，理性对待的程度，指数越高人越理性。",
				"id": 82,
				"knowledge": "什么是理性思维理性思维是一种有明确的思维方向，有充分的思维依据，能对事物或问题进行观察、比较、分析、综合、抽象与概括的一种思维。是一种建立在证据和逻辑推理基础上的思维方式。",
				"level": 2,
				"levelStr": "中",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 3,
				"status": 0,
				"type": "rai",
				"typeStr": "3理性指数",
				"updateBy": "13298598527",
				"updateTime": 1673855700000,
				"waveformRank": "13"
			},
			{
				"advise": "",
				"createBy": "13298598527",
				"createTime": 1673509746000,
				"description": "反映人的兴奋程度，指数越高人越兴奋。",
				"id": 83,
				"knowledge": "",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 4,
				"status": 0,
				"type": "exi",
				"typeStr": "4兴奋指数",
				"updateBy": "13298598527",
				"updateTime": 1673875397000,
				"waveformRank": "13"
			},
			{
				"advise": "",
				"createBy": "13298598527",
				"createTime": 1673509764000,
				"description": "反映人的抑郁程度，指数越高人越的抑郁程度越高。",
				"id": 84,
				"knowledge": "什么是抑郁抑郁是人的一种正常的负性情绪，是在人经历不好事情时候的一种情感体验。一般表现为：1、心境低落，主要表现为持久而显著的情感低落，抑郁，悲观，轻者闷闷不乐，无愉快感，兴趣减退；重者痛不欲生，悲观绝望，甚至有自杀的倾向或者自杀行为。2、思维迟钝，表现为思维联想速度缓慢，反应迟钝，思路闭塞。3、意志活动减退，呈持久显著的抑制，临床表现为行为缓慢，生活被动、懒散，不想做事，不愿意和周围人接触、交往，常常独自坐在一旁或整日卧床，闭门独居，回避社交；严重时，连吃、喝的生理需要和个人卫生都不顾，不修边幅。4、认知活动损害，对于近期事物记忆力下降，注意力障碍，反应时间延长。5、躯体症状，主要表现为睡眠障碍、乏力、食欲减退、体重下降、便秘，身体任何部分都可以出现疼痛、性欲减退、闭经等等。   一般情况下，人们在经历不好的事情之后，可能会出现这种抑郁的情绪，但一般不超过一周的时间。因为人们都有自我调节的能力，几天的时间可能抑郁的情绪就会明显的减轻甚至消失。如果超过两周以上的时间仍然没有明显的缓解，建议去医院进行诊治。",
				"level": 0,
				"levelStr": "无",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 5,
				"status": 0,
				"type": "yyzs",
				"typeStr": "5抑郁程度可能性",
				"updateBy": "13298598527",
				"updateTime": 1673855803000,
				"waveformRank": "13"
			},
			{
				"advise": "",
				"createBy": "13298598527",
				"createTime": 1673509778000,
				"description": "反映人被感觉或情绪等左右的程度，与理性相反，指数越高人越感性。",
				"id": 85,
				"knowledge": "",
				"level": 1,
				"levelStr": "弱",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 6,
				"status": 0,
				"type": "gxzs",
				"typeStr": "6感性指数",
				"updateBy": "13298598527",
				"updateTime": 1673875397000,
				"waveformRank": "13"
			}
		],
		"evaluateWaveformUnscrambles": [
			{
				"createBy": "admin",
				"createTime": 1688799348000,
				"id": 516,
				"label": "检测结果",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 1,
				"status": 0,
				"type": "consequence",
				"typeStr": "检测结果",
				"unscramble": "根据测评数据来看，您具备出色的组织管理能力和感统协调能力，同时也表现出了优秀的逻辑思维和分析能力。然而，您的精力似乎有比较大的消耗，可能是由于睡眠问题导致的。为了发挥您的潜力，建议您提高睡眠质量，以便更好地发挥您的能力和技能。此外，尽管您很努力，但在近期似乎出现了一些成就感不足和压力感较大的情况。您的兴奋度和幸福指数也有些偏低，这可能与您的睡眠和安排有关。为了增强工作和生活的乐趣，建议您调整关注点和时间安排，尝试拓宽社交关系并从中获得更多内在力量。让每一天都过得快乐，实现更高水平的表现和获得满满的幸福感。",
				"updateBy": "18942117941",
				"updateTime": 1688799348000,
				"waveformRank": "13",
				"δMaxLimit": 30,
				"δMinLimit": 20
			},
			{
				"createBy": "admin",
				"createTime": 1688799348000,
				"id": 515,
				"label": "对应建议",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 2,
				"status": 0,
				"type": "suggest",
				"typeStr": "对应建议",
				"unscramble": "从本次检测的数据来看，我们为您提供如下建议：首先，增加身体锻炼的次数和强度，这对于保持身体健康非常重要。如果可以的话，中午可以安排半小时左右的午休时间，或者通过冥想来放松身心。其次，我们需要调整好自己的心态，保持积极乐观的心态，这有助于改善睡眠质量。接着，我们要抓重要的事情，把握好“大局”，不能被琐事打乱了我们的节奏。同时，我们也需要放下束缚自己的想法，勇敢突破自己的个人局限，追求更高的目标。最后，我们需要放松自己，找到自己的乐趣，享受生活，更好的对待自己，我们要更好的照顾自己，提升身心健康，让每一天开心快乐。",
				"updateBy": "18942117941",
				"updateTime": 1688799348000,
				"waveformRank": "13",
				"δMaxLimit": 30,
				"δMinLimit": 20
			},
			{
				"createBy": "admin",
				"createTime": 1688799348000,
				"id": 514,
				"label": "解决方案",
				"maxAge": 1000,
				"minAge": 18,
				"params": {},
				"remark": "",
				"sex": 0,
				"sort": 3,
				"status": 0,
				"type": "solution",
				"typeStr": "解决方案",
				"unscramble": "详细方案请咨询测评解读师！",
				"updateBy": "18942117941",
				"updateTime": 1688799348000,
				"waveformRank": "13",
				"δMaxLimit": 30,
				"δMinLimit": 20
			}
		],
		"foreword": "EEG脑波生理信号：医疗科研级精准率达≥95% 脑电生物信号测评技术：是强脑公司（Brain Co）科研创新与具影响力的技术项目 （脑电生物反馈技术、脑电生物测评技术）之一，由权威测机构报告验证：EEG-单 极导联精准率达≥95%，是医疗科研级的技术前沿地位…… 双域双值-项目测评技术：根据脑波振荡频带动率（PSD) 主要是通过非入侵式--脑电生物技术，使用采集脑电生理信号，对额叶、顶叶、枕 叶、颞叶及大脑皮层的网络系统建立分析模型与科学算法，技术，解读当前（测评 前3、12、24小时乃至28天周期内）情绪、心理特征、中枢神经、植物神经系统、 内分泌等与体质免疫功能有关所发生的细胞生物电与脑电信号反应…… 2亿研发成本·70名科学家·5年时间中检集团权威测试报告",
		"getEegLog": {
			"a": 22.7,
			"d": 40.2,
			"g": 29.5,
			"h": 23.5,
			"l": 29,
			"t": 34,
			"timestrapStr": "null"
		},
		"scientificPrinciple": "哈佛大学创新实验室、哈佛大学脑科学中心 2015年BrainCo成功诞生于哈佛大学创新实验室，独立开发脑机接口设备。同年BrainCo采用一套 全新的自学习理论--“基于机器学习归类下的人类学习&quot;理论，成功研发出当时市场上精度最高， 加密型最好的可穿戴脑电芯片，以及可以如意操控的机械手等。 </p> 脑科学，狭义地讲就是神经科学，是为了了解神经系统内分子水平、细胞水平、细胞间的变化过程， 以及这些过程在中枢功能控制系统内的整合作用而进行的研究。(美国神经科学学会)广义的定义是 研究脑的结构和功能的科学，还包括认知神经科学等。 EEG脑电图应用 人们情绪、思想、行为形成的本质是大脑神经网络之间相互交流，大脑与身体大量神经元的生物电 “同步&quot;脉冲作用，产生了脑电波’,人体的神经系统(包括中枢神经、植物神经)和免疫系统产生交 互作用后，它们随即开始寻找生理上的连结并产生免疫应答。 脑波与健康应用 认知科学在脑机接口领域，有以BrainCo为代表的非侵入式和以 马斯克的Neurolink为代表的侵入式进行了认知神经科学的应用 研究，包括:知觉功能(视听嗅味触)，认知功能(记忆、学习、执行 等等)，社会情绪功能，心理抑制等，是认知神经科学测量方法， 已促进、推动科学发展与社会效应。其中作为有潜力的非侵入式 脑机接口技术有良好的时间分辨率、易用性、便捷性。PSD脑波情绪检测科学原理",
		"userInfo": {
			"userAge": 23,
			"userBirthday": 946656300000,
			"userHeight": 185,
			"userName": "张三",
			"userSex": 0,
			"userWeight": 150
		}
	},
` + mockDataTail

				//fmt.Println("mockData: ", mockData)
				// 创建一个新的请求实例
				req, err := http.NewRequest("POST", "https://127.0.0.1:8089/MentalClinic/data", bytes.NewBuffer([]byte(mockData)))
				if err != nil {
					fmt.Println("Error creating request:", err)
					return
				}

				req.Header.Set("Content-Type", "application/json")

				// 发起请求
				//client := &http.Client{}
				// 创建一个客户端，并配置自定义的Transport
				client := &http.Client{Transport: tr}

				resp, err := client.Do(req)
				if err != nil {
					fmt.Println("Error making request:", err)
					return
				}
				defer resp.Body.Close()

				// 读取响应体
				body, err := ioutil.ReadAll(resp.Body)
				if err != nil {
					fmt.Println("Error reading response:", err)
					return
				}

				// 打印响应状态和响应体
				fmt.Println("Response status:", resp.Status)
				fmt.Println("Response body:", string(body))

			}(UUID)

			for i := 0; i < 20; i++ { // 循环等待20次
				template := AppCache[UUID].UserInfoTemplate
				dpId := AppCache[UUID].DpId
				uid := AppCache[UUID].UserId

				if template != "" {

					/*
					  模拟用户登录
					*/
					//用户已经存在，模拟用户登录
					session := sessions.Default(c)
					//在这里修改,3表示有效时间为1800秒
					session.Options(sessions.Options{MaxAge: 1800})
					// 设置用户ID到会话中
					session.Set("UserID", utils.Strval(uid))
					if err := session.Save(); err != nil {
						// 如果保存Session失败，处理错误
						c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": "模拟登录失败"})
						return
					}
					var userId = session.Get("UserID").(string)
					fmt.Println(config.ColorPurple, "从session中获取userId: ", userId, config.ColorReset)
					config.WorldLoginCacheCode.Set(userId, true, time.Hour)

					// 构建响应数据
					response := gin.H{
						"code":           1,
						"callBackStatus": "获取数据成功!",
						"UserId":         uid,
						"UUID":           UUID,
						"DpId":           dpId,
					}

					// 序列化响应数据为 JSON 字符串
					jsonData, err := json.Marshal(response)
					if err != nil {
						// 处理序列化错误
						fmt.Println("序列化 JSON 数据出错:", err)
						return
					}
					// 打印 JSON 数据到控制台
					fmt.Println(string(jsonData))
					// 数据已到达，返回成功响应
					c.JSON(http.StatusOK, response)

					fmt.Println(config.ColorPurple, "取出缓存的UserId: ", uid, "取出缓存的dpId: ", dpId, config.ColorReset)
					return
				}

				fmt.Println("Response body:", string(body))

				time.Sleep(1 * time.Second) // 等待1秒钟后再次检查数据是否到达
			}

		} else if code == 500 || resp.StatusCode == 500 {
			c.JSON(http.StatusOK, gin.H{"code": 0, "message": "请求失败，请求ID已存在"})
		}
	}

}

// processData 是一个独立的函数，用于作为回调接收数据
func ProcessData(c *gin.Context) {

	var jsonData map[string]interface{}
	if err := c.BindJSON(&jsonData); err != nil {
		fmt.Printf("Error in BindJSON: %v\n", err)
		c.JSON(http.StatusBadRequest, gin.H{"error": err.Error()})
		return
	}
	// 获取 requestNo 的值
	requestNo, ok := jsonData["requestNo"].(string)
	if !ok {
		fmt.Println("Invalid requestNo value")
		c.JSON(http.StatusBadRequest, gin.H{"error": "Invalid requestNo value"})
		return
	}

	/*
	  判断uuid是否为请求时发送的UUID
	*/
	if AppCache[requestNo] == nil {
		c.JSON(http.StatusOK, gin.H{"code": 0, "message": "请求唯一识别码uuid不相等"})
		return
	}

	fmt.Println("requestNo: ", requestNo)
	// 打印接收到的数据
	//println(utils.Strval(jsonData))
	//for key, value := range jsonData {
	//	println("Received:", key, "=", utils.Strval(value))
	//}

	/*
	   将获取到的数据中的关于用户信息的数据保存到user_info表中
	*/
	jsonBytes, err := json.Marshal(jsonData)
	if err != nil {
		fmt.Printf("Error in Marshal: %v\n", err)
		c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
		return
	}
	//fmt.Println("jsonBytes >>> ", utils.Strval(jsonBytes))

	//从缓存中获取app的id和name，查询数据库application表中config_data配置信息
	appId := AppCache[requestNo].AppID
	appName := AppCache[requestNo].AppName

	configData, err := models.GetConfigDataByAppInfo(appId, appName)
	if err != nil {
		fmt.Printf("根据应用id和应用名称查询应用的配置信息失败")
		c.JSON(http.StatusOK, gin.H{"code": 0, "message": "根据应用id和应用名称查询应用的配置信息失败"})
		return
	}
	//从数据库中拿到配置数据后，遍历获取关键信息的映射关系
	values, err := GetKeyInfoMapping(jsonBytes, configData)
	if err != nil {
		fmt.Printf("遍历配置文件的关键信息映射路径信息失败")
		c.JSON(http.StatusOK, gin.H{"code": 0, "message": "遍历配置文件的关键信息映射路径信息失败"})
		return
	}
	fmt.Println("values :", values)
	/*
	   从configData中获取userInfoTemplate的字符串
	*/
	// 访问 map 中的 "userInfoTemplate" 键
	userInfoTemplate, ok := configData["userInfoTemplate"]
	if !ok {
		fmt.Println("Key 'userInfoTemplate' not found in the map")
		return
	}
	//访问map中的DpId键的值
	DpId, ok := configData["DpId"]
	if !ok {
		fmt.Println("Key 为DpId的键在map中未发现")
		return
	}
	DpIdStr, ok := DpId.(string)
	if !ok {
		fmt.Println("Dp_id is not a string")
		return
	}
	dp_id, err := strconv.ParseInt(DpIdStr, 10, 64)
	if err != nil {
		fmt.Println("从configData中获取的数字人id类型转换失败")
		c.JSON(http.StatusOK, gin.H{"code": 0, "message": "数字人id类型转换失败"})
		return
	}
	//将数字人id存入到缓存中
	AppCache[requestNo].DpId = dp_id
	fmt.Println(config.ColorPurple, "存入缓存的DpId: ", dp_id, config.ColorReset)

	// 断言 userInfoTemplate 为字符串类型
	userInfoTemplateStr, ok := userInfoTemplate.(string)
	if !ok {
		fmt.Println("userInfoTemplate is not a string")
		return
	}
	// 打印 userInfoTemplate 的值
	fmt.Println("userInfoTemplate: ", userInfoTemplateStr)
	/*
	   填充用户信息模板字符串中的占位符
	*/
	userInfoTemplateValue, err := FillUserInfoTemplate(userInfoTemplateStr, values)
	if err != nil {
		fmt.Println(" 填充用户信息模板字符串失败!")
		return
	}
	fmt.Println("userInfoTemplateValue", userInfoTemplateValue)

	//从缓存中获取手机号，学号和单位
	phone := AppCache[requestNo].Phone
	stuNo := AppCache[requestNo].SerialNo
	unit := AppCache[requestNo].Unit

	//将用户信息模板字符串保存至缓存中
	AppCache[requestNo].UserInfoTemplate = userInfoTemplateValue

	/*
	   录入用户数据到 用户表和用户详情信息表,不存在则创建，存在则修改
	*/
	var userName = values[0]
	var userSex = values[1]
	var UserHeight = values[2]
	var userWeight = values[3]
	var userBirthday = values[4]
	//开启事务
	session := database.Engine.NewSession()
	defer session.Close() //延迟关闭session
	err = session.Begin()
	if err != nil {
		c.JSON(http.StatusOK, gin.H{"code": 0, "message": "开启事务失败!"})
		fmt.Println("开启事务失败")
		return
	}
	/* 手机号不为空的情况 */
	if phone != "" {

		//根据手机号查询用户是否存在
		existUser, exist, err := models.GetUserByPhone(session, phone)
		if err != nil {
			fmt.Println("根据手机号查询用户是否存在，查询失败")
			c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": "根据手机号查询用户是否存在，查询失败"})
			return
		}
		//构建用户信息表对象
		user := &models.User{
			Phone:      phone,
			Createtime: time.Now().Unix(),
		}
		fmt.Println(config.ColorYellow, "username: ", userName, "phone: ", phone, config.ColorReset)
		//构建用户详情信息结构体对象
		userinfo := &models.UserInfo{
			UserSex:      userSex,
			Phone:        phone,
			UserHeight:   UserHeight,
			UserWeight:   userWeight,
			UserBirthday: userBirthday,
		}
		//若用户存在是否更新用户注册信息和用户详细信息
		IsUpdateSelfData := AppCache[requestNo].IsUpdateSelfData

		if !exist {
			//用户不存在，则用户名默认为手机号
			user.Username = phone
			//用户不存在，需要新创建用户，为用户设置初始密码加密
			initPassWord := "88888888"
			salt := config.Conf.Md5Salt
			passStr, err := utils.MD5BySalf(initPassWord, salt)
			if err != nil {
				log.Println("密码加盐失败！")
				c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": "密码加盐失败"})
				return
			}
			user.Password = passStr
			user.Status = 0
			user.Level = 9
			user_id, err := models.FindOrCreateOrUpdateUser(session, user)
			if err != nil {
				c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": "插入用户数据失败"})
				return
			}
			//将新录入的用户id保存到缓存中
			AppCache[requestNo].UserId = user_id
			fmt.Println(config.ColorPurple, "存入缓存的UserId: ", user_id, config.ColorReset)
			userinfo.UserId = user_id
			userinfo.UserName = userName
			//用户详情信息表操作
			err = models.FindOrCreateOrUpdateUserInfoByPhone(session, userinfo)
			if err != nil {
				fmt.Printf("向用户信息表添加用户数据失败")
				c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": err.Error()})
				return
			}

		} else {
			if IsUpdateSelfData == "true" {
				//用户已经存在
				user.Id = existUser.Id
				uid, err := models.UpdateUser_Tx(session, user)
				if err != nil {
					fmt.Printf("更新用户数据失败")
					c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": err.Error()})
					return
				}
				//将用户id保存到缓存中
				AppCache[requestNo].UserId = uid
				userinfo.UserId = uid
				userinfo.UserName = userName
				fmt.Println(config.ColorPurple, "存入缓存的UserId: ", uid, config.ColorReset)
				//创建或更新用户详细信息，如果用户详细信息存在则更新，不存在则创建
				err = models.CreateOrUpdateUserInfo(session, userinfo)
				if err != nil {
					fmt.Printf("更新用户详情信息失败!")
					c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": err.Error()})
					return
				}

			} else if IsUpdateSelfData == "false" {
				//不更新用户注册信息和详细信息
				user.Id = existUser.Id
				//将用户id保存到缓存中
				AppCache[requestNo].UserId = user.Id

			} else {
				/*
				  如果前端不传值，默认不修改用户注册信息和用户详情信息
				*/
				//不更新用户注册信息和详细信息
				user.Id = existUser.Id
				//将用户id保存到缓存中
				AppCache[requestNo].UserId = user.Id

			}

		}
		//c.JSON(http.StatusInternalServerError, gin.H{"code": 1, "message": "用户表和用户详情信息表数据录入成功"})
		fmt.Println(config.ColorGreen, "用户和用户详情信息数据录入成功", config.ColorReset)
		/* 提交事务操作 */
		err = session.Commit()
		if err != nil {
			c.JSON(http.StatusOK, gin.H{"code": 0, "message": "提交事务失败!"})
			fmt.Println("提交事务失败")
			return
		}
	}

	/*
	   录入用户数据到 用户表和用户详情信息表,不存在则创建，存在则修改
	*/
	//学号和学校单位不为空的情况
	if stuNo != "" && unit != "" {
		//若用户存在是否更新用户注册信息和用户详细信息
		IsUpdateSelfData := AppCache[requestNo].IsUpdateSelfData
		//构建用户详情信息结构体对象
		userinfo := &models.UserInfo{
			UserSex:      userSex,
			Unit:         unit,
			UserHeight:   UserHeight,
			UserWeight:   userWeight,
			UserBirthday: userBirthday,
		}
		userinfo.StudentId = stuNo
		//查询用户详情信息是否存在
		user_info, exist, err := models.FindUserByStudentIdAndUnit(session, userinfo)

		if err != nil {
			c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": "根据学号和单位名称查询用户信息失败!"})
			return
		}

		//如果详情用户信息表不存在该用户
		if !exist {
			//用户初始密码加密
			userinfo.UserName = userName
			initPassWord := "88888888"
			salt := config.Conf.Md5Salt
			passStr, err := utils.MD5BySalf(initPassWord, salt)
			if err != nil {
				log.Println("密码加盐失败！")
				c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": "密码加盐失败"})
				return
			}
			//这里没有手机号，因为手机号在表中为唯一索引，因此用手机号_学校名称的字符串存入
			CreatePhone := stuNo + "_" + unit
			//定义用户注册信息
			user := &models.User{
				Username:   stuNo,
				Createtime: time.Now().Unix(),
				Password:   passStr,
				Phone:      CreatePhone,
				Level:      9,
				Status:     0,
			}

			_, id, err := models.CreateAndUpdateUser(session, user)
			if err != nil {
				c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": "插入或更新用户数据失败!"})
				return
			}
			//用户详情信息表中不存在该用户的详情信息
			userinfo.UserId = id
			//将新录入的用户id保存到缓存中
			AppCache[requestNo].UserId = id
			fmt.Println(config.ColorPurple, "存入缓存的UserId: ", id, config.ColorReset)
			err = models.CreateUserInfo(session, userinfo)
			if err != nil {
				c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": "插入用户详情信息失败!"})
				return
			}

		} else {
			/*
			  如果用户详情信息表存在该用户，将用户id保存到缓存中
			*/
			if IsUpdateSelfData == "true" {
				//如果用户详情信息表存在该用户，将用户id保存到缓存中
				userinfo.UserName = userName
				AppCache[requestNo].UserId = user_info.UserId
				fmt.Println(config.ColorPurple, "存入缓存的UserId: ", user_info.UserId, config.ColorReset)
				//存在则更新
				//err = models.FindOrCreateOrUpdateUserInfo(user_info)
				err = models.UpdateUserInfo(session, user_info)
				if err != nil {
					fmt.Printf("更新用户详情信息失败!")
					c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": err.Error()})
					return
				}

			} else if IsUpdateSelfData == "false" {
				/*
				 不更新用户详情信息
				*/
				//如果用户详情信息表存在该用户，将用户id保存到缓存中
				AppCache[requestNo].UserId = user_info.UserId
				fmt.Println(config.ColorPurple, "存入缓存的UserId: ", user_info.UserId, config.ColorReset)

			} else {
				/*
				  如果前端flag没有传值则默认更新userinfo信息
				*/
				//如果用户详情信息表存在该用户，将用户id保存到缓存中
				AppCache[requestNo].UserId = user_info.UserId
				userinfo.UserName = userName
				fmt.Println(config.ColorPurple, "存入缓存的UserId: ", user_info.UserId, config.ColorReset)
				//存在则更新
				//err = models.FindOrCreateOrUpdateUserInfo(user_info)
				err = models.UpdateUserInfo(session, user_info)
				if err != nil {
					fmt.Printf("更新用户详情信息失败!")
					c.JSON(http.StatusInternalServerError, gin.H{"code": 0, "message": err.Error()})
					return
				}
			}

		}

		//c.JSON(http.StatusInternalServerError, gin.H{"code": 1, "message": "用户表和用户详情信息表数据录入成功"})
		fmt.Println(config.ColorGreen, "用户表和用户详情信息表数据录入成功", config.ColorReset)
		/* 提交事务操作 */
		err = session.Commit()
		if err != nil {
			c.JSON(http.StatusOK, gin.H{"code": 0, "message": "提交事务失败!"})
			fmt.Println("提交事务失败")
			return
		}
	}

	// 确保文件夹存在
	dirPath := "./mental_clinic_data/"
	if err := EnsureDir(dirPath); err != nil {
		fmt.Printf("Error in ensureDir: %v\n", err)
		c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
		return
	}

	// 生成文件名并写入文件
	filePath := filepath.Join(dirPath, GenerateFileName(requestNo))
	err = ioutil.WriteFile(filePath, jsonBytes, 0755)
	if err != nil {
		fmt.Printf("Error in WriteFile: %v\n", err)
		c.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":           1,
		"callBackStatus": "获取数据成功!",
		"message":        "用户表和用户详情信息表数据录入成功",
	})

}

// 生成文件名
func GenerateFileName(UUID string) string {
	return fmt.Sprintf("%s_%s.json", UUID, time.Now().Format("20060102150405"))
}

// 确保文件夹存在
func EnsureDir(dirName string) error {
	err := os.MkdirAll(dirName, os.ModePerm)
	if err != nil {
		return fmt.Errorf("error creating directory: %v", err)
	}
	return nil
}

//根据config配置，遍历映射关系
func GetKeyInfoMapping(jsonData []byte, configData map[string]interface{}) ([]string, error) {
	if configData == nil {
		return nil, fmt.Errorf("config data is nil")
	}

	keyInfoMapping, ok := configData["keyInfoMappingRelationship"]
	if !ok {
		return nil, fmt.Errorf("keyInfoMappingRelationship not found in config data")
	}

	keyInfoMappingArr, ok := keyInfoMapping.([]interface{})
	if !ok {
		return nil, fmt.Errorf("keyInfoMappingRelationship is not an array")
	}

	keyInfoMap := make([]string, len(keyInfoMappingArr))
	values := make([]string, len(keyInfoMappingArr))
	for i, val := range keyInfoMappingArr {
		strVal, ok := val.(string)
		if !ok {
			return nil, fmt.Errorf("element in keyInfoMappingRelationship is not a string")
		}
		keyInfoMap[i] = strVal
		readerValue, err := utils.JSONReader(jsonData, strVal)
		if err != nil {
			fmt.Println("获取值失败", err)
		}
		values[i] = utils.Strval(readerValue)
	}
	//生日处理
	// 将字符串转换为整数类型的时间戳
	intTimestamp, err := strconv.ParseInt(values[4], 10, 64)
	if err != nil {
		// 转换失败，处理错误
		fmt.Println("Error:", err)
		return nil, err
	}

	// 将时间戳转换为 time.Time 类型
	t := time.Unix(0, intTimestamp*int64(time.Millisecond))
	// 格式化为日期字符串
	dateString := t.Format("2006-01-02")
	values[4] = dateString
	//性别处理
	switch values[1] {
	case "0":
		values[1] = "男"
	case "1":
		values[1] = "女"
	default:
		values[1] = "未知"
	}

	return values, nil
}

// 填充用户信息模板
func FillUserInfoTemplate(template string, values []string) (string, error) {
	// 将 values 切片转换为 []interface{} 类型
	interfaceValues := make([]interface{}, len(values))
	for i, v := range values {
		interfaceValues[i] = v
	}

	// 使用 fmt.Sprintf 根据 values 填充模板
	filledTemplate := fmt.Sprintf(template, interfaceValues...)
	return filledTemplate, nil
}
